@@ -17,6 +17,7 @@
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
+#include <wchar.h>
 #include <time.h>
 #include <locale.h>
 #include <string.h>
@@ -29,12 +30,13 @@
 //#include <mgl/mgl_w.h>
 //-----------------------------------------------------------------------------
 #include "mgl/mgl_parse.h"
+extern mglTeXsymb mgl_tex_symb[];
 int test(mglGraphAB *gr)
 {
 	mglParse par;
 	par.AllowSetSize = true;
 	FILE *fp=fopen("test.mgl","rt");
-	par.Execute(gr,fp);
+	par.Execute(gr,fp,true);
 	fclose(fp);
 /*	gr->SetDrawReg(2,2,1);
 	gr->Rotate(40,60);
@@ -89,6 +91,8 @@ static struct option longopts[] =
 	{ "gif",			no_argument,	&type,		6 },
 	{ "none",			no_argument,	&type,		7 },
 	{ "bps",			no_argument,	&type,		8 },
+	{ "u3d",			no_argument,	&type,		9 },
+	{ "pdf",			no_argument,	&type,		10 },
 	{ "help",			no_argument,	NULL,		'?' },
 	{ NULL,				0,				NULL,		0 }
 };
@@ -102,6 +106,8 @@ void usage()
 		"-mini			- png picture is 200x133\n"		\
 		"-big			- png picture is 1200x800\n"	\
 		"-idtf			- output idtf\n"				\
+		"-u3d			- output u3d\n"				\
+		"-pdf			- output pdf\n"				\
 		"-eps			- output EPS\n"					\
 		"-jpeg			- output JPEG\n"				\
 		"-solid			- output solid PNG\n"			\
@@ -147,6 +153,12 @@ void save(mglGraph *gr,const char *name,const char *suf="")
 			sprintf(buf,"%s%s.png",name,suf);
 			gr->WritePNG(buf,0,false);
 			break;
+		case 9:	// U3D
+			sprintf(buf,"%s%s.u3d",name,suf);
+			gr->WriteU3D(buf);	break;
+		case 10:	// PDF
+			sprintf(buf,"%s%s.pdf",name,suf);
+			gr->WritePDF(buf);	break;
 		default:// PNG (no alpha)
 			sprintf(buf,"%s%s.png",name,suf);
 			gr->WritePNG(buf,0,false);	break;
@@ -181,6 +193,9 @@ void smgl_fonts(mglGraph *gr)	// ticks features
 //-----------------------------------------------------------------------------
 void smgl_stick(mglGraph *gr)	// column plot
 {
+	if(type==5 || type==9 || type==10)	
+		gr->Puts(mglPoint(0,0),"Does not work in 3D\\n for reasons I do not care to investigate till 2.0");
+
 	gr->SetRanges(-1, 1, -1, 1, 0, 1);	gr->Light(true);
 	gr->StickPlot(3, 0, 40, 30);		gr->Axis("xyz_");
 	gr->Surf("exp(-10*y^2-6*x^2)");
@@ -227,84 +242,95 @@ void smgl_map(mglGraph *gr)	// example of mapping
 	gr->Map(a, b, "brgk", 0, false);
 }
 //-----------------------------------------------------------------------------
+void smgl_color_schemes(mglGraph *gr)	// Color table
+{
+	mglData a(256,2);	a.Fill(-1,1);
+	gr->SubPlot(2,10,0,0.2);	gr->Dens(a,"kw");
+	gr->Puts(mglPoint(-1.4, -0.3), "kw", "C", -8);
+	gr->SubPlot(2,10,1,0.2);	gr->Dens(a,"wk");
+	gr->Puts(mglPoint(-1.4, -0.3), "wk", "C", -8);
+	gr->SubPlot(2,10,2,0.2);	gr->Dens(a,"kHCcw");
+	gr->Puts(mglPoint(-1.4, -0.3), "kHCcw", "C", -8);
+	gr->SubPlot(2,10,3,0.2);	gr->Dens(a,"kBbcw");
+	gr->Puts(mglPoint(-1.4, -0.3), "kBbcw", "C", -8);
+	gr->SubPlot(2,10,4,0.2);	gr->Dens(a,"kRryw");
+	gr->Puts(mglPoint(-1.4, -0.3), "kRryw", "C", -8);
+	gr->SubPlot(2,10,5,0.2);	gr->Dens(a,"kGgew");
+	gr->Puts(mglPoint(-1.4, -0.3), "kGgew", "C", -8);
+	gr->SubPlot(2,10,6,0.2);	gr->Dens(a,"BbwrR");
+	gr->Puts(mglPoint(-1.4, -0.3), "BbwrR", "C", -8);
+	gr->SubPlot(2,10,7,0.2);	gr->Dens(a,"BbwgG");
+	gr->Puts(mglPoint(-1.4, -0.3), "BbwgG", "C", -8);
+	gr->SubPlot(2,10,8,0.2);	gr->Dens(a,"GgwmM");
+	gr->Puts(mglPoint(-1.4, -0.3), "GgwmM", "C", -8);
+	gr->SubPlot(2,10,9,0.2);	gr->Dens(a,"UuwqR");
+	gr->Puts(mglPoint(-1.4, -0.3), "UuwqR", "C", -8);
+	gr->SubPlot(2,10,10,0.2);	gr->Dens(a,"QqwcC");
+	gr->Puts(mglPoint(-1.4, -0.3), "QqwcC", "C", -8);
+	gr->SubPlot(2,10,11,0.2);	gr->Dens(a,"CcwyY");
+	gr->Puts(mglPoint(-1.4, -0.3), "CcwyY", "C", -8);
+	gr->SubPlot(2,10,12,0.2);	gr->Dens(a,"bcwyr");
+	gr->Puts(mglPoint(-1.4, -0.3), "bcwyr", "C", -8);
+	gr->SubPlot(2,10,13,0.2);	gr->Dens(a,"bwr");
+	gr->Puts(mglPoint(-1.4, -0.3), "bwr", "C", -8);
+	gr->SubPlot(2,10,14,0.2);	gr->Dens(a,"BbcyrR");
+	gr->Puts(mglPoint(-1.4, -0.3), "BbcyrR", "C", -8);
+	gr->SubPlot(2,10,15,0.2);	gr->Dens(a,"UbcyqR");
+	gr->Puts(mglPoint(-1.4, -0.3), "UbcyqR", "C", -8);
+	gr->SubPlot(2,10,16,0.2);	gr->Dens(a,"BbcwyrR");
+	gr->Puts(mglPoint(-1.4, -0.3), "BbcwyrR", "C", -8);
+	gr->SubPlot(2,10,17,0.2);	gr->Dens(a,"bcyr");
+	gr->Puts(mglPoint(-1.4, -0.3), "bcyr", "C", -8);
+	gr->SubPlot(2,10,18,0.2);	gr->Dens(a,"BbcyrR|");
+	gr->Puts(mglPoint(-1.4, -0.3), "BbcyrR|", "C", -8);
+	gr->SubPlot(2,10,19,0.2);	gr->Dens(a,"bgr");
+	gr->Puts(mglPoint(-1.4, -0.3), "bgr", "C", -8);
+}
+//-----------------------------------------------------------------------------
 void smgl_colors(mglGraph *gr)	// Color table
 {
-	gr->Face(mglPoint(-1,  -1), mglPoint(-1,  -0.7), mglPoint(-0.6,-1), mglPoint(-0.6,-0.7), "L#");
-	gr->Puts(mglPoint(-0.8,-0.9, 0.01), "L", "C:w", -1.4);
-	gr->Face(mglPoint(-0.6,-1), mglPoint(-0.6,-0.7), mglPoint(-0.2,-1), mglPoint(-0.2,-0.7), "E#");
-	gr->Puts(mglPoint(-0.4,-0.9, 0.01), "E", "C:w", -1.4);
-	gr->Face(mglPoint(-0.2,-1), mglPoint(-0.2,-0.7), mglPoint(0.2,-1), mglPoint(0.2,-0.7), "N#");
-	gr->Puts(mglPoint(0,  -0.9, 0.01), "N", "C:w", -1.4);
-	gr->Face(mglPoint(0.2,-1), mglPoint(0.2,-0.7), mglPoint(0.6,-1), mglPoint(0.6,-0.7), "U#");
-	gr->Puts(mglPoint(0.4,-0.9, 0.01), "U", "C:w", -1.4);
-	gr->Face(mglPoint(0.6,-1), mglPoint(0.6,-0.7), mglPoint(1,  -1), mglPoint(1,  -0.7), "Q#");
-	gr->Puts(mglPoint(0.8,-0.9, 0.01), "Q", "C:w", -1.4);
+	//#LENUQ
+	gr->FaceZ(-1,	-1, 0, 0.4, 0.3, "L#");	gr->Puts(mglPoint(-0.8,-0.9), "L", "C:w", -1.4);
+	gr->FaceZ(-0.6,	-1, 0, 0.4, 0.3, "E#");	gr->Puts(mglPoint(-0.4,-0.9), "E", "C:w", -1.4);
+	gr->FaceZ(-0.2,	-1, 0, 0.4, 0.3, "N#");	gr->Puts(mglPoint(0,  -0.9), "N", "C:w", -1.4);
+	gr->FaceZ(0.2,	-1, 0, 0.4, 0.3, "U#");	gr->Puts(mglPoint(0.4,-0.9), "U", "C:w", -1.4);
+	gr->FaceZ(0.6,	-1, 0, 0.4, 0.3, "Q#");	gr->Puts(mglPoint(0.8,-0.9), "Q", "C:w", -1.4);
 	//#lenuq
-	gr->Face(mglPoint(-1,  -0.7), mglPoint(-1,  -0.4), mglPoint(-0.6,-0.7), mglPoint(-0.6,-0.4), "l#");
-	gr->Puts(mglPoint(-0.8,-0.6, 0.01), "l", "C:k", -1.4);
-	gr->Face(mglPoint(-0.6,-0.7), mglPoint(-0.6,-0.4), mglPoint(-0.2,-0.7), mglPoint(-0.2,-0.4), "e#");
-	gr->Puts(mglPoint(-0.4,-0.6, 0.01), "e", "C:k", -1.4);
-	gr->Face(mglPoint(-0.2,-0.7), mglPoint(-0.2,-0.4), mglPoint(0.2,-0.7), mglPoint(0.2,-0.4), "n#");
-	gr->Puts(mglPoint(0,  -0.6, 0.01), "n", "C:k", -1.4);
-	gr->Face(mglPoint(0.2,-0.7), mglPoint(0.2,-0.4), mglPoint(0.6,-0.7), mglPoint(0.6,-0.4), "u#");
-	gr->Puts(mglPoint(0.4,-0.6, 0.01), "u", "C:k", -1.4);
-	gr->Face(mglPoint(0.6,-0.7), mglPoint(0.6,-0.4), mglPoint(1,  -0.7), mglPoint(1,  -0.4), "q#");
-	gr->Puts(mglPoint(0.8,-0.6, 0.01), "q", "C:k", -1.4);
+	gr->FaceZ(-1,	-0.7, 0, 0.4, 0.3, "l#");	gr->Puts(mglPoint(-0.8,-0.6), "l", "C:k", -1.4);
+	gr->FaceZ(-0.6,	-0.7, 0, 0.4, 0.3, "e#");	gr->Puts(mglPoint(-0.4,-0.6), "e", "C:k", -1.4);
+	gr->FaceZ(-0.2,	-0.7, 0, 0.4, 0.3, "n#");	gr->Puts(mglPoint(0,  -0.6), "n", "C:k", -1.4);
+	gr->FaceZ(0.2,	-0.7, 0, 0.4, 0.3, "u#");	gr->Puts(mglPoint(0.4,-0.6), "u", "C:k", -1.4);
+	gr->FaceZ(0.6,	-0.7, 0, 0.4, 0.3, "q#");	gr->Puts(mglPoint(0.8,-0.6), "q", "C:k", -1.4);
 	//#CMYkP
-	gr->Face(mglPoint(-1,  -0.4), mglPoint(-1,  -0.1), mglPoint(-0.6,-0.4), mglPoint(-0.6,-0.1), "C#");
-	gr->Puts(mglPoint(-0.8,-0.3, 0.01), "C", "C:w", -1.4);
-	gr->Face(mglPoint(-0.6,-0.4), mglPoint(-0.6,-0.1), mglPoint(-0.2,-0.4), mglPoint(-0.2,-0.1), "M#");
-	gr->Puts(mglPoint(-0.4,-0.3, 0.01), "M", "C:w", -1.4);
-	gr->Face(mglPoint(-0.2,-0.4), mglPoint(-0.2,-0.1), mglPoint(0.2,-0.4), mglPoint(0.2,-0.1), "Y#");
-	gr->Puts(mglPoint(0,  -0.3, 0.01), "Y", "C:w", -1.4);
-	gr->Face(mglPoint(0.2,-0.4), mglPoint(0.2,-0.1), mglPoint(0.6,-0.4), mglPoint(0.6,-0.1), "k#");
-	gr->Puts(mglPoint(0.4,-0.3, 0.01), "k", "C:w", -1.4);
-	gr->Face(mglPoint(0.6,-0.4), mglPoint(0.6,-0.1), mglPoint(1,  -0.4), mglPoint(1,  -0.1), "P#");
-	gr->Puts(mglPoint(0.8,-0.3, 0.01), "P", "C:w", -1.4);
+	gr->FaceZ(-1,	-0.4, 0, 0.4, 0.3, "C#");	gr->Puts(mglPoint(-0.8,-0.3), "C", "C:w", -1.4);
+	gr->FaceZ(-0.6,	-0.4, 0, 0.4, 0.3, "M#");	gr->Puts(mglPoint(-0.4,-0.3), "M", "C:w", -1.4);
+	gr->FaceZ(-0.2,	-0.4, 0, 0.4, 0.3, "Y#");	gr->Puts(mglPoint(0,  -0.3), "Y", "C:w", -1.4);
+	gr->FaceZ(0.2,	-0.4, 0, 0.4, 0.3, "k#");	gr->Puts(mglPoint(0.4,-0.3), "k", "C:w", -1.4);
+	gr->FaceZ(0.6,	-0.4, 0, 0.4, 0.3, "P#");	gr->Puts(mglPoint(0.8,-0.3), "P", "C:w", -1.4);
 	//#cmywp
-	gr->Face(mglPoint(-1,  -0.1), mglPoint(-1,   0.2), mglPoint(-0.6,-0.1), mglPoint(-0.6, 0.2), "c#");
-	gr->Puts(mglPoint(-0.8, 0, 0.01), "c", "C:k", -1.4);
-	gr->Face(mglPoint(-0.6,-0.1), mglPoint(-0.6, 0.2), mglPoint(-0.2,-0.1), mglPoint(-0.2, 0.2), "m#");
-	gr->Puts(mglPoint(-0.4, 0, 0.01), "m", "C:k", -1.4);
-	gr->Face(mglPoint(-0.2,-0.1), mglPoint(-0.2, 0.2), mglPoint(0.2,-0.1), mglPoint(0.2, 0.2), "y#");
-	gr->Puts(mglPoint(0,   0, 0.01), "y", "C:k", -1.4);
-	gr->Face(mglPoint(0.2,-0.1), mglPoint(0.2, 0.2), mglPoint(0.6,-0.1), mglPoint(0.6, 0.2), "w#");
-	gr->Puts(mglPoint(0.4, 0, 0.01), "w", "C:k", -1.4);
-	gr->Face(mglPoint(0.6,-0.1), mglPoint(0.6, 0.2), mglPoint(1,  -0.1), mglPoint(1,   0.2), "p#");
-	gr->Puts(mglPoint(0.8, 0, 0.01), "p", "C:k", -1.4);
+	gr->FaceZ(-1,	-0.1, 0, 0.4, 0.3, "c#");	gr->Puts(mglPoint(-0.8, 0), "c", "C:k", -1.4);
+	gr->FaceZ(-0.6,	-0.1, 0, 0.4, 0.3, "m#");	gr->Puts(mglPoint(-0.4, 0), "m", "C:k", -1.4);
+	gr->FaceZ(-0.2,	-0.1, 0, 0.4, 0.3, "y#");	gr->Puts(mglPoint(0,   0), "y", "C:k", -1.4);
+	gr->FaceZ(0.2,	-0.1, 0, 0.4, 0.3, "w#");	gr->Puts(mglPoint(0.4, 0), "w", "C:k", -1.4);
+	gr->FaceZ(0.6,	-0.1, 0, 0.4, 0.3, "p#");	gr->Puts(mglPoint(0.8, 0), "p", "C:k", -1.4);
 	//#BGRHW
-	gr->Face(mglPoint(-1,   0.2), mglPoint(-1,   0.5), mglPoint(-0.6, 0.2), mglPoint(-0.6, 0.5), "B#");
-	gr->Puts(mglPoint(-0.8, 0.3, 0.01), "B", "C:w", -1.4);
-	gr->Face(mglPoint(-0.6, 0.2), mglPoint(-0.6, 0.5), mglPoint(-0.2, 0.2), mglPoint(-0.2, 0.5), "G#");
-	gr->Puts(mglPoint(-0.4, 0.3, 0.01), "G", "C:w", -1.4);
-	gr->Face(mglPoint(-0.2, 0.2), mglPoint(-0.2, 0.5), mglPoint(0.2, 0.2), mglPoint(0.2, 0.5), "R#");
-	gr->Puts(mglPoint(0,   0.3, 0.01), "R", "C:w", -1.4);
-	gr->Face(mglPoint(0.2, 0.2), mglPoint(0.2, 0.5), mglPoint(0.6, 0.2), mglPoint(0.6, 0.5), "H#");
-	gr->Puts(mglPoint(0.4, 0.3, 0.01), "H", "C:w", -1.4);
-	gr->Face(mglPoint(0.6, 0.2), mglPoint(0.6, 0.5), mglPoint(1,   0.2), mglPoint(1,   0.5), "W#");
-	gr->Puts(mglPoint(0.8, 0.3, 0.01), "W", "C:w", -1.4);
+	gr->FaceZ(-1,	0.2, 0, 0.4, 0.3, "B#");	gr->Puts(mglPoint(-0.8, 0.3), "B", "C:w", -1.4);
+	gr->FaceZ(-0.6,	0.2, 0, 0.4, 0.3, "G#");	gr->Puts(mglPoint(-0.4, 0.3), "G", "C:w", -1.4);
+	gr->FaceZ(-0.2,	0.2, 0, 0.4, 0.3, "R#");	gr->Puts(mglPoint(0,   0.3), "R", "C:w", -1.4);
+	gr->FaceZ(0.2,	0.2, 0, 0.4, 0.3, "H#");	gr->Puts(mglPoint(0.4, 0.3), "H", "C:w", -1.4);
+	gr->FaceZ(0.6,	0.2, 0, 0.4, 0.3, "W#");	gr->Puts(mglPoint(0.8, 0.3), "W", "C:w", -1.4);
 	//#bgrhw
-	gr->Face(mglPoint(-1,   0.5), mglPoint(-1,   0.8), mglPoint(-0.6, 0.5), mglPoint(-0.6, 0.8), "b#");
-	gr->Puts(mglPoint(-0.8, 0.6, 0.01), "b", "C:k", -1.4);
-	gr->Face(mglPoint(-0.6, 0.5), mglPoint(-0.6, 0.8), mglPoint(-0.2, 0.5), mglPoint(-0.2, 0.8), "g#");
-	gr->Puts(mglPoint(-0.4, 0.6, 0.01), "g", "C:k", -1.4);
-	gr->Face(mglPoint(-0.2, 0.5), mglPoint(-0.2, 0.8), mglPoint(0.2, 0.5), mglPoint(0.2, 0.8), "r#");
-	gr->Puts(mglPoint(0,   0.6, 0.01), "r", "C:k", -1.4);
-	gr->Face(mglPoint(0.2, 0.5), mglPoint(0.2, 0.8), mglPoint(0.6, 0.5), mglPoint(0.6, 0.8), "h#");
-	gr->Puts(mglPoint(0.4, 0.6, 0.01), "h", "C:k", -1.4);
-	gr->Face(mglPoint(0.6, 0.5), mglPoint(0.6, 0.8), mglPoint(1,   0.5), mglPoint(1,   0.8), "w#");
-	gr->Puts(mglPoint(0.8, 0.6, 0.01), "w", "C:k", -1.4);
+	gr->FaceZ(-1,	0.5, 0, 0.4, 0.3, "b#");	gr->Puts(mglPoint(-0.8, 0.6), "b", "C:k", -1.4);
+	gr->FaceZ(-0.6,	0.5, 0, 0.4, 0.3, "g#");	gr->Puts(mglPoint(-0.4, 0.6), "g", "C:k", -1.4);
+	gr->FaceZ(-0.2,	0.5, 0, 0.4, 0.3, "r#");	gr->Puts(mglPoint(0,   0.6), "r", "C:k", -1.4);
+	gr->FaceZ(0.2,	0.5, 0, 0.4, 0.3, "h#");	gr->Puts(mglPoint(0.4, 0.6), "h", "C:k", -1.4);
+	gr->FaceZ(0.6,	0.5, 0, 0.4, 0.3, "w#");	gr->Puts(mglPoint(0.8, 0.6), "w", "C:k", -1.4);
 	//#brighted
-	gr->Face(mglPoint(-1,   0.8), mglPoint(-1,   1.1), mglPoint(-0.6, 0.8), mglPoint(-0.6, 1.1), "r1#");
-	gr->Puts(mglPoint(-0.8, 0.9, 0.01), "r1", "C:w", -1.4);
-	gr->Face(mglPoint(-0.6, 0.8), mglPoint(-0.6, 1.1), mglPoint(-0.2, 0.8), mglPoint(-0.2, 1.1), "r3#");
-	gr->Puts(mglPoint(-0.4, 0.9, 0.01), "r3", "C:w", -1.4);
-	gr->Face(mglPoint(-0.2, 0.8), mglPoint(-0.2, 1.1), mglPoint(0.2, 0.8), mglPoint(0.2, 1.1), "r5#");
-	gr->Puts(mglPoint(0,   0.9, 0.01), "r5", "C:k", -1.4);
-	gr->Face(mglPoint(0.2, 0.8), mglPoint(0.2, 1.1), mglPoint(0.6, 0.8), mglPoint(0.6, 1.1), "r7#");
-	gr->Puts(mglPoint(0.4, 0.9, 0.01), "r7", "C:k", -1.4);
-	gr->Face(mglPoint(0.6, 0.8), mglPoint(0.6, 1.1), mglPoint(1,   0.8), mglPoint(1,   1.1), "r9#");
-	gr->Puts(mglPoint(0.8, 0.9, 0.01), "r9", "C:k", -1.4);
+	gr->FaceZ(-1,	0.8, 0, 0.4, 0.3, "r1#");	gr->Puts(mglPoint(-0.8, 0.9), "r1", "C:w", -1.4);
+	gr->FaceZ(-0.6,	0.8, 0, 0.4, 0.3, "r3#");	gr->Puts(mglPoint(-0.4, 0.9), "r3", "C:w", -1.4);
+	gr->FaceZ(-0.2,	0.8, 0, 0.4, 0.3, "r5#");	gr->Puts(mglPoint(0,   0.9), "r5", "C:k", -1.4);
+	gr->FaceZ(0.2,	0.8, 0, 0.4, 0.3, "r7#");	gr->Puts(mglPoint(0.4, 0.9), "r7", "C:k", -1.4);
+	gr->FaceZ(0.6,	0.8, 0, 0.4, 0.3, "r9#");	gr->Puts(mglPoint(0.8, 0.9), "r9", "C:k", -1.4);
 }
 //-----------------------------------------------------------------------------
 void smgl_qo2d(mglGraph *gr)
@@ -341,7 +367,7 @@ void smgl_pde(mglGraph *gr)	// PDE and Ray sample
 	a.Transpose("yxz");
 	gr->CAxis(0, 1);
 	gr->Dens(a,"wyrRk");
-	gr->Plot("-x", "k|", type==5?0.01:NAN);
+	gr->Plot("-x", "k|", (type==5 || type==9 || type==10)?0.01:NAN);
 	gr->Puts(mglPoint(0, 0.85), "absorption: (x+z)/2 for x+z>0");
 	gr->Title("Equation: ik_0\\partial_zu + \\Delta u + x\\cdot u + i \\frac{x+z}{2}\\cdot u = 0", "C", -float(width)/height);
 	gr->Compression(false);  //put setting back
@@ -454,6 +480,7 @@ void smgl_samplec(mglGraph *gr)	// error boxes
 	for(int i=0;i<10;i++)
 		gr->Error(mglPoint(mgl_rnd()-0.5,mgl_rnd()-0.5,mgl_rnd()-0.5), mglPoint(0.1,0.1,0.1),"bo");
 	gr->Axis();
+	gr->Rotate(0,0); // for unrotate in IDTF
 }
 //-----------------------------------------------------------------------------
 void smgl_sampleb(mglGraph *gr)	// Gaussian beam
@@ -536,7 +563,7 @@ void smgl_sample8(mglGraph *gr)	// 1d plot
 	mglData x(50);		x.Modify("cos(pi*2*x-pi)");
 	gr->Plot(x,y0,"Y+");
 
-	gr->Plot2(y1,"q|");
+	gr->Plot(y1.SubData(-1,0),y1.SubData(-1,1),"q|");
 
 	gr->SubPlot(2,2,2);	gr->Rotate(60,40);
 	mglData z(50);		z.Modify("2*x-1");
@@ -544,10 +571,12 @@ void smgl_sample8(mglGraph *gr)	// 1d plot
 
 	mglData y2(10,3);	y2.Modify("cos(pi*(2*x-1-y))");
 	y2.Modify("2*x-1",2);
-	gr->Plot3(y2,"bo ");
+	gr->Plot(y2.SubData(-1,0),y2.SubData(-1,1),y2.SubData(-1,2),"bo ");
 
 	gr->SubPlot(2,2,3);	gr->Rotate(60,40);
 	gr->Bars(x,y0,z,"ri");		gr->Box();
+	
+	gr->Rotate(0,0); // for unrotate in IDTF
 }
 //-----------------------------------------------------------------------------
 void smgl_sample7(mglGraph *gr)	// smoothing
@@ -587,24 +616,43 @@ void smgl_sample6(mglGraph *gr)	// differentiate
 //-----------------------------------------------------------------------------
 void smgl_sample5(mglGraph *gr)	// pen styles
 {
-	if(type==5)	gr->Puts(mglPoint(0,1.2),"line styles not supported","rL");
-	gr->Line(mglPoint(0,1,0),mglPoint(0.3,1,0),"k-");	gr->Puts(mglPoint(0.4,1),"Solid '-'","rL");
-	gr->Line(mglPoint(0,0.7,0),mglPoint(0.3,0.7,0),"k|");	gr->Puts(mglPoint(0.4,0.7),"Dash '|'","rL");
-	gr->Line(mglPoint(0,0.4,0),mglPoint(0.3,0.4,0),"k;");	gr->Puts(mglPoint(0.4,0.4),"Small dash ';'","rL");
-	gr->Line(mglPoint(0,0.1,0),mglPoint(0.3,0.1,0),"kj");	gr->Puts(mglPoint(0.4,0.1),"Dash-dot 'j'","rL");
-	gr->Line(mglPoint(0,-0.2,0),mglPoint(0.3,-0.2,0),"ki");	gr->Puts(mglPoint(0.4,-0.2),"Small dash-dot 'i'","rL");
-	gr->Line(mglPoint(0,-0.5,0),mglPoint(0.3,-0.5,0),"k:");	gr->Puts(mglPoint(0.4,-0.5),"Dots ':'","rL");
-	gr->Line(mglPoint(0,-0.8,0),mglPoint(0.3,-0.8,0),"k ");	gr->Puts(mglPoint(0.4,-0.8),"None ' '","rL");
+	if(type==5 || type==9 || type==10)	gr->Puts(mglPoint(0,1.2),"line styles not supported","rL");
+	mreal d,x1,x2,x0,y=0.95;
+	d=0.3, x0=0.2, x1=0.5, x2=0.6;
+	gr->Line(mglPoint(x0,1-0*d),mglPoint(x1,1-0*d),"k-");	gr->Puts(mglPoint(x2,y-0*d),"Solid '-'","rL");
+	gr->Line(mglPoint(x0,1-1*d),mglPoint(x1,1-1*d),"k|");	gr->Puts(mglPoint(x2,y-1*d),"Long Dash '|'","rL");
+	gr->Line(mglPoint(x0,1-2*d),mglPoint(x1,1-2*d),"k;");	gr->Puts(mglPoint(x2,y-2*d),"Dash ';'","rL");
+	gr->Line(mglPoint(x0,1-3*d),mglPoint(x1,1-3*d),"k=");	gr->Puts(mglPoint(x2,y-3*d),"Small dash '='","rL");
+	gr->Line(mglPoint(x0,1-4*d),mglPoint(x1,1-4*d),"kj");	gr->Puts(mglPoint(x2,y-4*d),"Dash-dot 'j'","rL");
+	gr->Line(mglPoint(x0,1-5*d),mglPoint(x1,1-5*d),"ki");	gr->Puts(mglPoint(x2,y-5*d),"Small dash-dot 'i'","rL");
+	gr->Line(mglPoint(x0,1-6*d),mglPoint(x1,1-6*d),"k:");	gr->Puts(mglPoint(x2,y-6*d),"Dots ':'","rL");
+	gr->Line(mglPoint(x0,1-7*d),mglPoint(x1,1-7*d),"k ");	gr->Puts(mglPoint(x2,y-7*d),"None ' '","rL");
 
-	gr->Mark(mglPoint(-1,1.2,0),'.');	gr->Puts(mglPoint(-0.7,1.2),"'.'","rL");
-	gr->Mark(mglPoint(-1,0.9,0),'+');	gr->Puts(mglPoint(-0.7,0.9),"'+'","rL");
-	gr->Mark(mglPoint(-1,0.6,0),'x');	gr->Puts(mglPoint(-0.7,0.6),"'x'","rL");
-	gr->Mark(mglPoint(-1,0.3,0),'*');	gr->Puts(mglPoint(-0.7,0.3),"'*'","rL");
-	gr->Mark(mglPoint(-1,0.0,0),'s');	gr->Puts(mglPoint(-0.7,0.0),"'s'","rL");
-	gr->Mark(mglPoint(-1,-0.3,0),'d');	gr->Puts(mglPoint(-0.7,-0.3),"'d'","rL");
-	gr->Mark(mglPoint(-1,-0.6,0),'o');	gr->Puts(mglPoint(-0.7,-0.6),"'o'","rL");
-	gr->Mark(mglPoint(-1,-0.9,0),'^');	gr->Puts(mglPoint(-0.7,-0.9),"'\\^'","rL");
-	gr->Mark(mglPoint(-1,-1.2,0),'v');	gr->Puts(mglPoint(-0.7,-1.2),"'v'","rL");
+	d=0.25; x1=-1; x0=-0.8;	y = -0.05;
+	gr->Mark(mglPoint(x1,5*d),'.');		gr->Puts(mglPoint(x0,y+5*d),"'.'","rL");
+	gr->Mark(mglPoint(x1,4*d),'+');		gr->Puts(mglPoint(x0,y+4*d),"'+'","rL");
+	gr->Mark(mglPoint(x1,3*d),'x');		gr->Puts(mglPoint(x0,y+3*d),"'x'","rL");
+	gr->Mark(mglPoint(x1,2*d),'*');		gr->Puts(mglPoint(x0,y+2*d),"'*'","rL");
+	gr->Mark(mglPoint(x1,d),'s');		gr->Puts(mglPoint(x0,y+d),"'s'","rL");
+	gr->Mark(mglPoint(x1,0),'d');		gr->Puts(mglPoint(x0,y),"'d'","rL");
+	gr->Mark(mglPoint(x1,-d,0),'o');	gr->Puts(mglPoint(x0,y-d),"'o'","rL");
+	gr->Mark(mglPoint(x1,-2*d,0),'^');	gr->Puts(mglPoint(x0,y-2*d),"'\\^'","rL");
+	gr->Mark(mglPoint(x1,-3*d,0),'v');	gr->Puts(mglPoint(x0,y-3*d),"'v'","rL");
+	gr->Mark(mglPoint(x1,-4*d,0),'<');	gr->Puts(mglPoint(x0,y-4*d),"'<'","rL");
+	gr->Mark(mglPoint(x1,-5*d,0),'>');	gr->Puts(mglPoint(x0,y-5*d),"'>'","rL");
+
+	d=0.25; x1=-0.5; x0=-0.3;	y = -0.05;
+	gr->Mark(mglPoint(x1,5*d),'C');		gr->Puts(mglPoint(x0,y+5*d),"'\\#.'","rL");
+	gr->Mark(mglPoint(x1,4*d),'P');		gr->Puts(mglPoint(x0,y+4*d),"'\\#+'","rL");
+	gr->Mark(mglPoint(x1,3*d),'X');		gr->Puts(mglPoint(x0,y+3*d),"'\\#x'","rL");
+	gr->Mark(mglPoint(x1,2*d),'Y');		gr->Puts(mglPoint(x0,y+2*d),"'\\#*'","rL");
+	gr->Mark(mglPoint(x1,d),'S');		gr->Puts(mglPoint(x0,y+d),"'\\#s'","rL");
+	gr->Mark(mglPoint(x1,0),'D');		gr->Puts(mglPoint(x0,y),"'\\#d'","rL");
+	gr->Mark(mglPoint(x1,-d,0),'O');	gr->Puts(mglPoint(x0,y-d),"'\\#o'","rL");
+	gr->Mark(mglPoint(x1,-2*d,0),'T');	gr->Puts(mglPoint(x0,y-2*d),"'\\#\\^'","rL");
+	gr->Mark(mglPoint(x1,-3*d,0),'V');	gr->Puts(mglPoint(x0,y-3*d),"'\\#v'","rL");
+	gr->Mark(mglPoint(x1,-4*d,0),'L');	gr->Puts(mglPoint(x0,y-4*d),"'\\#<'","rL");
+	gr->Mark(mglPoint(x1,-5*d,0),'R');	gr->Puts(mglPoint(x0,y-5*d),"'\\#>'","rL");
 }
 //-----------------------------------------------------------------------------
 void smgl_sample4(mglGraph *gr)	// font features
@@ -700,6 +748,8 @@ void smgl_sample2(mglGraph *gr)	// axis and grid
 	gr->Label('z',"Z axis",0);
 	gr->Puts(mglPoint(0,0,1.5),"Remove X axis, and");
 	gr->Puts(mglPoint(0,0,1.2),"decrease number of ticks");
+	
+	gr->Rotate(0,0); // for unrotate in IDTF
 }
 //-----------------------------------------------------------------------------
 void smgl_sample1(mglGraph *gr)	// transformation
@@ -707,7 +757,7 @@ void smgl_sample1(mglGraph *gr)	// transformation
 	gr->SubPlot(2,2,0);  // just new axis without rotation and aspects
 	gr->Box();
 	gr->Text(mglPoint(-1,1.1,1),"Just box","rL");
-	gr->InPlot(0.2,0.5,0.7,1);
+	gr->InPlot(0.2,0.5,0.7,1,false);
 	gr->Box();
 	gr->Text(mglPoint(0,1.2,1),"InPlot example");
 	gr->SubPlot(2,2,1);  // new axis with aspect and rotation
@@ -725,6 +775,7 @@ void smgl_sample1(mglGraph *gr)	// transformation
 	gr->Aspect(1,2,2);
 	gr->Box();
 	gr->Text(mglPoint(0,0,1.5),"Aspect in other direction");
+	gr->Rotate(0,0); // for unrotate in IDTF
 }
 //-----------------------------------------------------------------------------
 void mgls_prepare1d(mglData *y, mglData *y1=0, mglData *y2=0, mglData *x1=0, mglData *x2=0)
@@ -762,6 +813,12 @@ void smgl_area(mglGraph *gr)
 	gr->Org=mglPoint();	gr->Box();	gr->Area(y);
 }
 //-----------------------------------------------------------------------------
+void smgl_area_2(mglGraph *gr)
+{
+	mglData y;	mgls_prepare1d(&y);
+	gr->Org=mglPoint();	gr->Box();	gr->Area(y,"cbgGyr");
+}
+//-----------------------------------------------------------------------------
 void smgl_stem(mglGraph *gr)
 {
 	mglData y;	mgls_prepare1d(&y);
@@ -774,6 +831,24 @@ void smgl_step(mglGraph *gr)
 	gr->Box();	gr->Step(y);
 }
 //-----------------------------------------------------------------------------
+void smgl_bars_2(mglGraph *gr)
+{
+	mglData ys(10,3);	ys.Modify("0.8*sin(pi*(2*x+y/2))+0.2*rnd");
+	gr->Org=mglPoint();	gr->Box();	gr->Bars(ys,"cbgGyr");
+}
+//-----------------------------------------------------------------------------
+void smgl_bars_a(mglGraph *gr)
+{
+	mglData ys(10,3);	ys.Modify("0.3*sin(pi*(2*x+y/2))+0.1*rnd");
+	gr->Org=mglPoint();	gr->Box();	gr->Bars(ys,"a");
+}
+//-----------------------------------------------------------------------------
+void smgl_bars_f(mglGraph *gr)
+{
+	mglData ys(10,2);	ys.Modify("0.24*sin(pi*(2*x+y/2))+0.06*rnd");
+	gr->Org=mglPoint();	gr->Box();	gr->Bars(ys,"f");
+}
+//-----------------------------------------------------------------------------
 void smgl_bars(mglGraph *gr)
 {
 	mglData ys(10,3);	ys.Modify("0.8*sin(pi*(2*x+y/2))+0.2*rnd");
@@ -807,6 +882,13 @@ void smgl_region(mglGraph *gr)
 	gr->Plot(y1,"k2");	gr->Plot(y2,"k2");
 }
 //-----------------------------------------------------------------------------
+void smgl_region_2(mglGraph *gr)
+{
+	mglData y1,y2;	mgls_prepare1d(0, &y1, &y2);
+	gr->Box();	gr->Region(y2,y1,"yr");
+	gr->Plot(y1,"k2");	gr->Plot(y2,"k2");
+}
+//-----------------------------------------------------------------------------
 void smgl_mark(mglGraph *gr)
 {
 	mglData y,y1;	mgls_prepare1d(&y,&y1);
@@ -875,6 +957,14 @@ void smgl_chart(mglGraph *gr)
 	gr->Box();	gr->Chart(ch,"#");
 }
 //-----------------------------------------------------------------------------
+void smgl_ring_chart(mglGraph *gr)
+{
+	mglData ch(7,2);	for(int i=0;i<7*2;i++)	ch.a[i]=mgl_rnd()+0.1;
+	gr->Light(true);	gr->Rotate(40,60);	gr->VertexColor(false);
+	gr->SetFunc("(y+2)/3*cos(pi*x)","(y+2)/3*sin(pi*x)","");
+	gr->Box();	gr->Chart(ch,"bgr cmy#");
+}
+//-----------------------------------------------------------------------------
 void smgl_pie_chart(mglGraph *gr)
 {
 	mglData ch(7,2);	for(int i=0;i<7*2;i++)	ch.a[i]=mgl_rnd()+0.1;
@@ -897,6 +987,13 @@ void mgls_prepare2d(mglData *a, mglData *b=0, mglData *v=0)
 	}
 }
 //-----------------------------------------------------------------------------
+void smgl_grad(mglGraph *gr)
+{
+	mglData a;	mgls_prepare2d(&a);
+	gr->Box();	gr->Grad(a);
+	gr->Alpha(true);	gr->Dens(a);
+}
+//-----------------------------------------------------------------------------
 void smgl_contt(mglGraph *gr)
 {
 	mglData a;	mgls_prepare2d(&a);
@@ -935,6 +1032,17 @@ void smgl_surf(mglGraph *gr)
 	gr->Box();	gr->Surf(a);
 }
 //-----------------------------------------------------------------------------
+void smgl_stereo(mglGraph *gr)
+{
+	mglData a;	mgls_prepare2d(&a);
+	gr->Light(true);
+	gr->SubPlot(2,1,0);	gr->Rotate(40,60+3);
+	gr->Box();	gr->Surf(a);
+	gr->SubPlot(2,1,1);	gr->Rotate(40,60-3);
+	gr->Box();	gr->Surf(a);
+	gr->Rotate(0,0); // for unrotate in IDTF
+}
+//-----------------------------------------------------------------------------
 void smgl_tile(mglGraph *gr)
 {
 	mglData a;	mgls_prepare2d(&a);
@@ -995,7 +1103,7 @@ void smgl_surf_fog(mglGraph *gr)
 {
 	mglData a;	mgls_prepare2d(&a);
 	gr->Light(true);	gr->Rotate(40,60);	gr->Fog(1);	gr->Box();
-	type==5?gr->Text(mglPoint(),"Fog not supported") : gr->Surf(a);
+	(type==5 || type==9 || type==10)?gr->Text(mglPoint(),"Fog not supported") : gr->Surf(a);
 	gr->Fog(0);
 }
 //-----------------------------------------------------------------------------
@@ -1117,7 +1225,7 @@ void smgl_cloudp(mglGraph *gr)
 	mglData c;	mgls_prepare3d(&c);
 	gr->Rotate(40,60);	gr->Alpha(true);	gr->VertexColor(false);
 	gr->Box();
-	if(type==5)	gr->Text(mglPoint(0,0,0),"Point clouds not supported");
+	if(type==5 || type==9 || type==10)	gr->Text(mglPoint(0,0,0),"Point clouds not supported");
 	else		gr->CloudP(c,"wyrRk");
 }
 //-----------------------------------------------------------------------------
@@ -1273,14 +1381,14 @@ void smgl_legend(mglGraph *gr)
 	}
 	gr->Axis(mglPoint(0), mglPoint(1));
 	gr->Box();	gr->Plot(f);	gr->Axis("xy");
-	if(type==5)	gr->LegendBox = false;
+	if(type==5 || type==9 || type==10)	gr->LegendBox = false;
 	gr->AddLegend("sin(\\pi {x^2})","b");	gr->AddLegend("sin(\\pi x)","g*");
 	gr->AddLegend("sin(\\pi \\sqrt{x})","r+");	gr->Legend();
 }
 //-----------------------------------------------------------------------------
 void smgl_type0(mglGraph *gr)	// TranspType = 0
 {
-	if(type==5)	return;
+	if(type==5 || type==9 || type==10)	return;
 	gr->Alpha(true);	gr->Light(true);	gr->Light(0,mglPoint(0,0,1));
 	mglData a;	mgls_prepare2d(&a);
 	gr->TranspType = 0;	gr->Clf();
@@ -1292,7 +1400,7 @@ void smgl_type0(mglGraph *gr)	// TranspType = 0
 //-----------------------------------------------------------------------------
 void smgl_type1(mglGraph *gr)	// TranspType = 1
 {
-	if(type==5)	return;
+	if(type==5 || type==9 || type==10)	return;
 	gr->Alpha(true);	gr->Light(true);	gr->Light(0,mglPoint(0,0,1));
 	mglData a;	mgls_prepare2d(&a);
 	gr->TranspType = 1;	gr->Clf();
@@ -1304,7 +1412,7 @@ void smgl_type1(mglGraph *gr)	// TranspType = 1
 //-----------------------------------------------------------------------------
 void smgl_type2(mglGraph *gr)	// TranspType = 2
 {
-	if(type==5)	return;
+	if(type==5 || type==9 || type==10)	return;
 	gr->Alpha(true);	gr->Light(true);	gr->Light(0,mglPoint(0,0,1));
 	mglData a;	mgls_prepare2d(&a);
 	gr->TranspType = 2;	gr->Clf();
@@ -1536,7 +1644,7 @@ void smgl_flow_dens(mglGraph *gr)	// flow threads and density plot
 {
 	mglData a,b,d;	mgls_prepare2v(&a,&b);	d = a;
 	for(int i=0;i<a.nx*a.ny;i++)	d.a[i] = hypot(a.a[i],b.a[i]);
-	gr->Flow(a,b,"br",5,true,type==5?-0.99:NAN);
+	gr->Flow(a,b,"br",5,true,(type==5 || type==9 || type==10)?-0.99:NAN);
 	gr->Dens(d,"BbcyrR");	gr->Box();
 }
 //-----------------------------------------------------------------------------
@@ -1668,16 +1776,28 @@ int main(int argc,char **argv)
 			default:	usage();	return 0;
 		}
 
-	if(type==5)			gr = &u3d;
-	else if(type==1 || type==2 || type==8)	gr = &ps;
+	if(type==5 || type==9 || type==10)
+	{
+		u3d.unrotate_flag = true;
+		gr = &u3d;
+	}
+	else 
+	{
+		if(type==1 || type==2 || type==8)	gr = &ps;
 	else				gr = &zb;
+	}
+
 	if(mini)		{	gr->SetSize(200,133);	suf = "_sm";	}
 	else if(big)
 	{	gr->SetSize(1200,800);	suf = "_lg";	gr->BaseLineWidth = 2;	}
 	else	gr->SetSize(width,height);
 
 	if(dotest)
-	{	test(gr);	gr->WritePNG("test.png","",false);	return 0;	}
+	{
+		mglTestMode=true;	test(gr);
+		gr->WriteEPS("test.eps");	gr->WritePNG("test.png","",false);
+		return 0;
+	}
 
 	if(srnd)	mgl_srnd(1);
 	gr->VertexColor(false);	gr->TextureColor(true);	gr->Compression(false);
@@ -1693,7 +1813,11 @@ int main(int argc,char **argv)
 		int i=0;
 		for(i=0;samp[i].name[0];i++);	// determine the number of samples
 		s = (mglSample *) bsearch(&tst, samp, i, sizeof(mglSample), mgl_cmd_smp);
-		if(s)	{	s->func(gr);	save(gr, s->name, suf);	}
+		if(s)
+		{
+			gr->DefaultPlotParam();	gr->Clf();
+			s->func(gr);	save(gr, s->name, suf);
+		}
 		else	printf("no sample %s\n",name);
 	}
 	printf("\n");	return 0;
@@ -1702,15 +1826,20 @@ int main(int argc,char **argv)
 mglSample samp[] = {
 	{"2_axis", smgl_2_axis},
 	{"area", smgl_area},
+	{"area_2", smgl_area_2},
 	{"axial", smgl_axial},
 	{"barh", smgl_barh},
 	{"bars", smgl_bars},
+	{"bars_2", smgl_bars_2},
+	{"bars_a", smgl_bars_a},
+	{"bars_f", smgl_bars_f},
 	{"belt", smgl_belt},
 	{"boxplot", smgl_boxplot},
 	{"boxs", smgl_boxs},
 	{"chart", smgl_chart},
 	{"cloud", smgl_cloud},
 	{"cloudp", smgl_cloudp},
+	{"color_schemes", smgl_color_schemes},
 	{"colors", smgl_colors},
 	{"column",	smgl_column},
 	{"cont", smgl_cont},
@@ -1739,6 +1868,7 @@ mglSample samp[] = {
 	{"flow3", smgl_flow3},
 	{"flow_dens", smgl_flow_dens},
 	{"fonts", smgl_fonts},
+	{"grad", smgl_grad},
 	{"legend", smgl_legend},
 	{"loglog", smgl_loglog},
 	{"map",		smgl_map},
@@ -1757,6 +1887,8 @@ mglSample samp[] = {
 	{"qo2d", smgl_qo2d},
 	{"radar", smgl_radar},
 	{"region", smgl_region},
+	{"region_2", smgl_region_2},
+	{"ring_chart", smgl_ring_chart},
 	{"sample1", smgl_sample1},
 	{"sample2", smgl_sample2},
 	{"sample3", smgl_sample3},
@@ -1776,6 +1908,7 @@ mglSample samp[] = {
 	{"sew", smgl_sew},
 	{"stem", smgl_stem},
 	{"step", smgl_step},
+	{"stereo", smgl_stereo},
 	{"stfa", smgl_stfa},
 	{"stick", smgl_stick},
 	{"surf", smgl_surf},
