@@ -17,6 +17,9 @@
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
+#ifdef _MGL_W_H_
+#warning "MathGL wrapper was enabled. So disable original MathGL classes"
+#else
 #ifndef _MGL_DATA_H_
 #define _MGL_DATA_H_
 //-----------------------------------------------------------------------------
@@ -34,16 +37,6 @@ struct gsl_matrix;
 #endif
 #include "mgl/mgl_define.h"
 //-----------------------------------------------------------------------------
-/// Class for drawing in windows (like, mglGraphFLTK, mglGraphQT and so on)
-/// Make inherited class and redefine Draw() function if you don't want to use function pointers.
-class mglGraph;
-struct mglDraw
-{
-	virtual int Draw(mglGraph *)	{	return 0;	};
-	virtual void Reload(int)	{};
-	virtual ~mglDraw()	{};
-};
-//-----------------------------------------------------------------------------
 /// Class for incapsulating point in space
 struct mglPoint
 {
@@ -80,6 +73,9 @@ inline mreal Norm(const mglPoint &p)
 /// Class for working with data array
 class mglData
 {
+protected:
+	/// Read data array from HDF4 file
+	void ReadHDF4(const char *fname,const char *data);
 public:
 
 	long nx;		///< number of points in 1st dimensions ('x' dimension)
@@ -136,10 +132,12 @@ public:
 	/// Rearange data dimensions
 	void Rearrange(int mx, int my=0, int mz=0);
 
-	/// Read data array from HDF file
+	/// Read data array from HDF file (parse HDF4 and HDF5 files)
 	void ReadHDF(const char *fname,const char *data);
 	/// Save data to HDF file
 	void SaveHDF(const char *fname,const char *data,bool rewrite=false) const;
+	/// Put HDF data names into buf as '\t' separated.
+	int DatasHDF(const char *fname, char *buf, long size);
 	/// Read data from tab-separated text file with auto determining size
 	bool Read(const char *fname);
 	/// Read data from text file with specifeid size
@@ -181,6 +179,8 @@ public:
 	void Fill(const char *eq, mglPoint r1, mglPoint r2, const mglData *v=0, const mglData *w=0);
 	/// Eqidistantly fill the data to range [x1,x2] in direction \a dir
 	void Fill(mreal x1,mreal x2,char dir='x');
+	/// Fill data by 'x'/'k' samples for Hankel ('h') or Fourier ('f') transform
+	void FillSample(int n, const char *how);
 	/// Get column (or slice) of the data filled by formulas of other named columns
 	mglData Column(const char *eq);
 	/// Set names for columns (slices)
@@ -270,9 +270,18 @@ public:
 	void Diff2(const char *dir);
 	/// Swap left and right part of the data in given direction (useful for fourier spectrums)
 	void Swap(const char *dir);
+	/// Roll data along direction \a dir by \a num slices
+	void Roll(char dir, int num);
 	/// Mirror the data in given direction (useful for fourier spectrums)
 	void Mirror(const char *dir);
 
+	/// Hankel transform
+	void Hankel(const char *dir);
+	/// Sin-Fourier transform
+	void SinFFT(const char *dir);
+	/// Cos-Fourier transform
+	void CosFFT(const char *dir);
+
 	/// Interpolate by 5-th order splain the data to given point \a x,\a y,\a z which normalized in range [0, 1] and evaluate its derivatives
 	mreal Spline5(mreal x,mreal y,mreal z,mreal &dx,mreal &dy,mreal &dz) const;
 	/// Interpolate by qubic splain the data to given point x=[0...nx-1], y=[0...ny-1], z=[0...nz-1]
@@ -362,5 +371,9 @@ mglData mglRay(const char *ham, mglPoint r0, mglPoint p0, mreal dt=0.1, mreal tm
 mglData mglJacobian(const mglData &x, const mglData &y);
 /// Calculate Jacobian determinant for D{x(u,v,w), y(u,v,w), z(u,v,w)}
 mglData mglJacobian(const mglData &x, const mglData &y, const mglData &z);
+// Do something like Delone triangulation
+mglData mglTriangulation(const mglData &x, const mglData &y, const mglData &z, mreal er);
+mglData mglTriangulation(const mglData &x, const mglData &y, mreal er);
 //-----------------------------------------------------------------------------
 #endif
+#endif
