@@ -27,14 +27,19 @@
 
 #include <math.h>
 
-#ifdef WIN32
+#define MGL_VERSION	11.0
+
+//#ifdef WIN32
+#ifdef _MSC_VER
+#define chdir	_chdir // BORLAND has chdir
 #include <float.h>
 const unsigned long mgl_nan[2] = {0xffffffff, 0x7fffffff};
 #define NANd	(*(double*)mgl_nan)
 #define NANf	(*(float*)&(mgl_nan[1]))
+#if(MGL_USE_DOUBLE==1)
+#define NAN		NANd
+#else
 #define NAN		NANd
-#if defined(_MSC_VER)
-#define chdir	_chdir // BORLAND has chdir
 #endif
 #endif
 
@@ -60,6 +65,14 @@ typedef double mreal;
 typedef float mreal;
 #endif
 //-----------------------------------------------------------------------------
+#ifndef MGL_CMAP_COLOR
+#define MGL_CMAP_COLOR	32
+#endif
+//-----------------------------------------------------------------------------
+#ifndef MGL_DEF_VIEWER
+#define MGL_DEF_VIEWER "evince"
+#endif
+//-----------------------------------------------------------------------------
 #define mgl_min(a,b)	(((a)>(b)) ? (b) : (a))
 #define mgl_max(a,b)	(((a)>(b)) ? (a) : (b))
 //-----------------------------------------------------------------------------
@@ -93,6 +106,7 @@ enum{	// types of predefined curvelinear coordinate systems
 #define MGL_VEC_END	4	// draw vector to point
 #define MGL_VEC_MID	8	// draw vector with point at center
 #define MGL_VEC_DOT	16	// draw dot instead of vector
+#define MGL_VEC_GRD	32	// enable color gradient along vector/dash
 //-----------------------------------------------------------------------------
 enum{	// Codes for warnings/messages
 	mglWarnNone = 0,// Everything OK
@@ -110,6 +124,7 @@ enum{	// Codes for warnings/messages
 	mglWarnLId,		// Light: ID is out of range
 	mglWarnSize,	// Setsize: size(s) is zero or negative
 	mglWarnFmt,		// Format is not supported for that build
+	mglWarnTern,	// Axis ranges are incompatible
 	mglWarnEnd		// Maximal number of warnings (must be last)
 };
 //-----------------------------------------------------------------------------
@@ -131,6 +146,9 @@ struct mglThread
 };
 /// Start several thread for the task
 mreal mglStartThread(void *(*func)(void *), mreal (*fres)(mreal,mreal), int n, mreal *z, long *p, void *v);
+/// write testing messages
+extern bool mglTestMode;
+void mglTest(const char *str, ...);
 #ifndef MGL_NTH_DEF
 #define MGL_NTH_DEF		2
 #endif
@@ -140,5 +158,15 @@ mreal mglStartThread(void *(*func)(void *), mreal (*fres)(mreal,mreal), int n, m
 #endif
 extern int mglNumThr;		///< Number of thread for plotting and data handling
 //-----------------------------------------------------------------------------
+/// Class for drawing in windows (like, mglGraphFLTK, mglGraphQT and so on)
+/// Make inherited class and redefine Draw() function if you don't want to use function pointers.
+class mglGraph;
+struct mglDraw
+{
+	virtual int Draw(mglGraph *)	{	return 0;	};
+	virtual void Reload(int)	{};
+	virtual ~mglDraw()	{};
+};
+//-----------------------------------------------------------------------------
 #endif
 //-----------------------------------------------------------------------------
