@@ -18,6 +18,9 @@
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
 //-----------------------------------------------------------------------------
+#ifdef _MGL_W_H_
+#warning "MathGL wrapper was enabled. So disable original MathGL classes"
+#else
 #ifndef _MGL_GLUT_H_
 #define _MGL_GLUT_H_
 #include "mgl/mgl_gl.h"
@@ -54,4 +57,5 @@ private:
 };
 //-----------------------------------------------------------------------------
 #endif
+#endif
 //-----------------------------------------------------------------------------
