@@ -20,6 +20,7 @@
 #ifdef _MGL_DATA_H_
 #warning "MathGL wrapper is disabled due to using original MathGL classes"
 #else
+#ifndef _MGL_W_H_
 #define _MGL_W_H_
 
 #ifndef NO_GSL
@@ -152,6 +153,7 @@ public:
 	{	mgl_data_modify_vw(a,eq,v.a, w.a);	};
 	inline void Modify(const char *eq,const mglData &v){	mgl_data_modify_vw(a,eq,v.a, NULL);	};
 	inline void Fill(double x1,double x2,char dir='x')	{	mgl_data_fill(a,x1,x2,dir);	};
+	inline void FillSample(int n, const char *how)	{	mgl_data_fill_sample(a,n,how);	};
 	inline mglData Column(const char *eq)	{	return mglData(mgl_data_column(a,eq));	};
 	inline void SetColumnId(const char *ids)			{	mgl_data_set_id(a,ids);	};
 	inline void Squeeze(int rx,int ry=1,int rz=1,bool smooth=false)
@@ -211,8 +213,13 @@ public:
 	{	mgl_data_diff_par(a,v1.a,v2.a,v3.a);	};
 	inline void Diff2(const char *dir)		{	mgl_data_diff2(a,dir);	};
 	inline void Swap(const char *dir)		{	mgl_data_swap(a,dir);	};
-	inline void Mirror(const char *dir)	{	mgl_data_mirror(a,dir);	};
+	inline void Roll(char dir, int num)		{	mgl_data_roll(a,dir,num);	};
+	inline void Mirror(const char *dir)		{	mgl_data_mirror(a,dir);	};
 
+	inline void Hankel(const char *dir)	{	mgl_data_hankel(a,dir);	};
+	inline void SinFFT(const char *dir)	{	mgl_data_sinfft(a,dir);	};
+	inline void CosFFT(const char *dir)	{	mgl_data_cosfft(a,dir);	};
+	
 	inline double Spline(double x,double y=0,double z=0)
 	{	return mgl_data_spline(a,x,y,z);	};
 	inline double Spline1(double x,double y=0,double z=0)
@@ -286,6 +293,7 @@ public:
 	~mglGraph()	{	mgl_delete_graph(self);	};
 	inline HMGL Self()	{	return self;	};
 
+	inline int  GetWarn()	{	return mgl_get_warn(self);	};
 	inline void DefaultPlotParam()	{	mgl_set_def_param(self);	};
 	inline void SetPalColor(int n, float r, float g, float b)
 	{	if(n>=0 && n<100)	mgl_set_pal_color(self, n, r, g, b);	};
@@ -315,12 +323,13 @@ public:
 	inline void SetPlotFactor(float val)	{	mgl_set_plotfactor(self, val);	};
 	inline void SetDrawFace(bool enable)	{	mgl_set_draw_face(self, enable);};
 	inline void SetLegendBox(bool enable)	{	mgl_set_legend_box(self, enable);	};
+	inline void SetLegendMarks(int num)		{	mgl_set_legend_marks(self, num);	};
 	inline void LoadFont(const char *name, const char *path=NULL)
 	{	mgl_load_font(self, name, path);	};
 	inline void CopyFont(mglGraph *gr)		{	mgl_copy_font(self, gr->Self());};
 	inline void RestoreFont()				{	mgl_restore_font(self);	};
 
-	inline void ShowImage(const char *viewer="kuickshow", bool keep=0)
+	inline void ShowImage(const char *viewer=0, bool keep=0)
 	{	mgl_show_image(self, viewer, keep);	};
 	inline void WriteFrame(const char *fname,const char *descr="")
 	{	mgl_write_frame(self, fname, descr);	};
@@ -346,7 +355,7 @@ public:
 	inline void ResetFrames()	{	mgl_reset_frames(self);	};
 	inline void Flush()			{	mgl_flush(self);	};
 
-	void GetRGB(char *imgdata, int imglen)
+	inline void GetRGB(char *imgdata, int imglen)
 	{
 		int w=mgl_get_width(self);
 		int h=mgl_get_height(self);
@@ -414,11 +423,13 @@ public:
 	{	mgl_clf_rgb(self, r, g, b);	};
 	inline void SubPlot(int nx,int ny,int m, float dx=0, float dy=0)
 	{	mgl_subplot_d(self, nx, ny, m, dx, dy);	};
-	inline void InPlot(float x1,float x2,float y1,float y2, bool rel=false)
+	inline void SubPlot(int nx,int ny,int m, const char *style)
+	{	mgl_subplot_s(self, nx, ny, m, style);	};
+	inline void InPlot(float x1,float x2,float y1,float y2, bool rel=true)
 	{	if(rel)	mgl_inplot(self, x1, x2, y1, y2);
 		else	mgl_relplot(self, x1, x2, y1, y2);	};
-	inline void ColumnPlot(int num, int ind)
-	{	mgl_columnplot(self,num,ind);	};
+	inline void ColumnPlot(int num, int ind, float d=0)
+	{	mgl_columnplot_d(self,num,ind,d);	};
 	inline void StickPlot(int num, int i, float tet, float phi)
 	{	mgl_stickplot(self,num,i,tet,phi);	};
 	inline void Aspect(float Ax,float Ay,float Az)
@@ -459,13 +470,13 @@ public:
 
 	inline void Box(const char *col="k", bool ticks=true)
 	{	mgl_box_str(self, col, ticks);	};
-	inline void Axis(const char *dir="xyz", bool adjust=false)
+	inline void Axis(const char *dir="xyzt", bool adjust=false)
 	{	if(adjust)	mgl_adjust_ticks(self,dir);	mgl_axis(self, dir);	};
-	inline void Grid(const char *dir="xyz",const char *pen="B")
+	inline void Grid(const char *dir="xyzt",const char *pen="B")
 	{	mgl_axis_grid(self, dir, pen);	};
 	inline void Label(char dir, const char *text, float pos=+1, float size=-1.4, float shift=0)
 	{	mgl_label_ext(self, dir, text, pos, size, shift);	};
-	inline void Label(char dir, const wchar_t *text, float pos=+1, float size=-1.4, float shift=0)
+	inline void Label(char dir, const wchar_t *text, float pos=0, float size=-1.4, float shift=0)
 	{	mgl_labelw_ext(self, dir, text, pos, size, shift);	};
 	void Label(double x, double y, const char *text, const char *fnt=0, double size=-1.4)
 	{	mgl_label_xy(self,x,y,text,fnt,size);	}
@@ -562,10 +573,10 @@ public:
 	{	mgl_tens(self, y.a, c.a, pen);	};
 	inline void Area(mglData &x, mglData &y, mglData &z, const char *pen="")
 	{	mgl_area_xyz(self, x.a, y.a, z.a, pen);	};
-	inline void Area(mglData &x, mglData &y, const char *pen="", bool sum=false)
-	{	if(sum)	mgl_area_xy(self, x.a, y.a, pen);	else	mgl_area_xys(self, x.a, y.a, pen);	};
-	inline void Area(mglData &y, const char *pen="", bool sum=false)
-	{	if(sum)	mgl_area(self, y.a, pen);	else	mgl_area_s(self, y.a, pen);	};
+	inline void Area(mglData &x, mglData &y, const char *pen="")
+	{	mgl_area_xy(self, x.a, y.a, pen);	};
+	inline void Area(mglData &y, const char *pen="")
+	{	mgl_area(self, y.a, pen);	};
 	inline void Stem(mglData &x, mglData &y, mglData &z, const char *pen="")
 	{	mgl_stem_xyz(self, x.a, y.a, z.a, pen);	};
 	inline void Stem(mglData &x, mglData &y, const char *pen="")
@@ -799,6 +810,13 @@ public:
 	inline void FlowP(float x0, float y0, float z0, mglData &ax, mglData &ay, mglData &az, const char *sch="")
 	{	mgl_flowp_3d(self, x0, y0, z0, ax.a, ay.a, az.a, sch);	};
 
+	inline void Grad(const mglData &x, const mglData &y, const mglData &z, const mglData &phi, const char *sch=0, int num=3)
+	{	mgl_grad_xyz(self,x.a,y.a,z.a,phi.a,sch,num);	}
+	inline void Grad(const mglData &x, const mglData &y, const mglData &phi, const char *sch=0, int num=5, mreal zVal=NAN)
+	{	mgl_grad_xy(self,x.a,y.a,phi.a,sch,num,zVal);	}
+	inline void Grad(const mglData &phi, const char *sch=0, int num=5, mreal zVal=NAN)
+	{	mgl_grad(self,phi.a,sch,num,zVal);	}
+
 	inline void Pipe(mglData &x, mglData &y, mglData &ax, mglData &ay, const char *sch="", float r0=0.05, int num=5, bool central=true, float zVal=NAN)
 	{	mgl_pipe_xy(self, x.a, y.a, ax.a, ay.a, sch, r0, num, central, zVal);	};
 	inline void Pipe(mglData &ax, mglData &ay, const char *sch="", float r0=0.05, int num=5, bool central=true, float zVal=NAN)
@@ -876,6 +894,22 @@ public:
 	{	mgl_triplot_xyz(self, nums.a, x.a, y.a, z.a, sch);	};
 	inline void TriPlot(mglData &nums, mglData &x, mglData &y, const char *sch="", float zVal=NAN)
 	{	mgl_triplot_xy(self, nums.a, x.a, y.a, sch, zVal);	};
+	inline void QuadPlot(mglData &nums, mglData &x, mglData &y, mglData &z, mglData &c, const char *sch="")
+	{	mgl_quadplot_xyzc(self, nums.a, x.a, y.a, z.a, c.a, sch);	};
+	inline void QuadPlot(mglData &nums, mglData &x, mglData &y, mglData &z, const char *sch="")
+	{	mgl_quadplot_xyz(self, nums.a, x.a, y.a, z.a, sch);	};
+	inline void QuadPlot(mglData &nums, mglData &x, mglData &y, const char *sch="", float zVal=NAN)
+	{	mgl_quadplot_xy(self, nums.a, x.a, y.a, sch, zVal);	};
+
+	inline void TriCont(const mglData &nums, const mglData &x, const mglData &y, const mglData &z, const char *sch=0,int num=7,mreal zVal=NAN)
+	{	mgl_tricont_xyz(self, nums.a, x.a, y.a, z.a, sch, num, zVal);	};
+	inline void TriContV(const mglData &v, const mglData &nums, const mglData &x, const mglData &y, const mglData &z, const char *sch=0,mreal zVal=NAN)
+	{	mgl_tricont_xyzv(self, v.a, nums.a, x.a, y.a, z.a, sch, zVal);	};
+	inline void TriCont(const mglData &nums, const mglData &x, const mglData &y, const mglData &z, const mglData &a, const char *sch=0,int num=7,mreal zVal=NAN)
+	{	mgl_tricont_xyzc(self, nums.a, x.a, y.a, z.a, a.a, sch, num, zVal);	};
+	inline void TriContV(const mglData &v, const mglData &nums, const mglData &x, const mglData &y, const mglData &z, const mglData &a, const char *sch=0,mreal zVal=NAN)
+	{	mgl_tricont_xyzcv(self, v.a, nums.a, x.a, y.a, z.a, a.a, sch, zVal);	};
+
 	inline void Dots(mglData &x, mglData &y, mglData &z, const char *sch="")
 	{	mgl_dots(self, x.a, y.a, z.a, sch);	};
 	inline void Dots(mglData &x, mglData &y, mglData &z, mglData &a, const char *sch="")
@@ -990,6 +1024,7 @@ public:
 	{	return mgl_parse(gr->Self(), self, str, pos);	};
 	inline int Parse(mglGraph *gr, const wchar_t *str, int pos)
 	{	return mgl_parsew(gr->Self(), self, str, pos);	};
+	inline void ScanFunc(const wchar_t *line){	mgl_scan_func(self, line);	};
 	inline void Execute(mglGraph *gr, const char *str)
 	{	mgl_parse_text(gr->Self(), self, str);	};
 	inline void Execute(mglGraph *gr, const wchar_t *str)
@@ -999,3 +1034,4 @@ public:
 };
 //-----------------------------------------------------------------------------
 #endif
+#endif
