@@ -37,14 +37,24 @@ mglGraphAB::mglGraphAB(int w,int h) : mglGraph()
 	G = 0;	UseLight = false;	st_pos=-1;
 	memset(stack,0,MGL_STACK_ENTRY*13*sizeof(mreal));
 	SetSize(w,h);	ClfOnUpdate = true;	SetDrawReg(1,1,0);
-	AutoClf=true;	Delay = 100;	ObjId=0;
+	AutoClf=true;	Delay = 1;	ObjId=0;	pPos=0;
 	NoAutoFactor = false;	ShowMousePos = true;
 	BDef[0] = BDef[1] = BDef[2] = BDef[3] = 255;
 	DefaultPlotParam();
 }
 //-----------------------------------------------------------------------------
 mglGraphAB::~mglGraphAB()
-{	if(G)	{	delete []G;	delete []G4;	}	}
+{	if(G)	{	delete []G;	delete []G4;	delete []OI;	}	}
+//-----------------------------------------------------------------------------
+void mglGraphAB::DefaultPlotParam()
+{
+	mglGraph::DefaultPlotParam();
+}
+//-----------------------------------------------------------------------------
+void mglGraphAB::Rotate(mreal TetX,mreal TetZ,mreal TetY)
+{
+	mglGraph::Rotate( TetX,TetZ,TetY );
+}
 //-----------------------------------------------------------------------------
 void mglGraphAB::Pop()
 {
@@ -92,11 +102,12 @@ void mglGraphAB::Pen(mglColor col, char style,mreal width)
 	switch(style)
 	{
 	case '-': PDef = 0xffff;	break;
-	case '|': PDef = 0xff00;	break;
-	case ';': PDef = 0xf0f0;	break;
-	case ':': PDef = 0x8888;	break;
-	case 'j': PDef = 0xfe10;	break;
-	case 'i': PDef = 0xe4e4;	break;
+	case '|': PDef = 0x00ff;	break;
+	case ';': PDef = 0x0f0f;	break;
+	case '=': PDef = 0x3333;	break;
+	case ':': PDef = 0x1111;	break;
+	case 'j': PDef = 0x087f;	break;
+	case 'i': PDef = 0x2727;	break;
 	case ' ': PDef = 0x0000;	break;
 //	case 0:	  PDef = 0x0000;	break;
 //	default:  PDef = 0xffff;
@@ -132,21 +143,8 @@ void mglGraphAB::RotateN(mreal Tet,mreal x,mreal y,mreal z)
 void mglGraphAB::Perspective(mreal a)	// I'm too lazy for using 4*4 matrix
 {	Persp = fabs(a)/Depth;	}
 //-----------------------------------------------------------------------------
-void mglGraphAB::RestoreM()
-{
-	memcpy(B,BL,9*sizeof(mreal));
-	xPos = BL[9];	yPos = BL[10];	zPos = BL[11];
-	PlotFactor = BL[12];
-}
-//-----------------------------------------------------------------------------
 void mglGraphAB::InPlot(mreal x1,mreal x2,mreal y1,mreal y2, bool rel)
 {
-	if(!rel)
-	{
-		memcpy(BL,B,9*sizeof(mreal));
-		BL[9] = xPos;	BL[10] = yPos;	BL[11] = zPos;
-		BL[12] = PlotFactor;
-	}
 	SelectPen("k-1");
 	if(Width<=0 || Height<=0 || Depth<=0)	return;
 	memset(B,0,9*sizeof(mreal));
@@ -305,10 +303,11 @@ void mglGraphAB::Light(int n, bool enable)
 	nLight[n] = enable;
 }
 //-----------------------------------------------------------------------------
-void mglGraphAB::Light(int n, mglPoint p, mglColor c, mreal br, bool inf)
+void mglGraphAB::Light(int n, mglPoint p, mglColor c, mreal br, bool inf, mreal ap)
 {
 	if(n<0 || n>9)	{	SetWarn(mglWarnLId);	return;	}
-	nLight[n] = true;	aLight[n] = 3;	bLight[n] = br;	iLight[n] = inf;
+	nLight[n] = true;	aLight[n] = ap>0?ap*ap:3;
+	bLight[n] = br;		iLight[n] = inf;
 	rLight[3*n] = p.x;	rLight[3*n+1] = p.y;	rLight[3*n+2] = p.z;
 	cLight[3*n] = c.r;	cLight[3*n+1] = c.g;	cLight[3*n+2] = c.b;
 }
@@ -370,23 +369,24 @@ void mglGraphAB::Putsw(mglPoint p, const wchar_t *wcs, const char *font, mreal s
 	if(font && strchr(font, 'a'))	{	Labelw(p.x, p.y, wcs,font,size,true);	return;	}
 
 	wchar_t *wcl;
-	int wn=0;
+	unsigned wn=0;
 	const wchar_t *wnl=0;
 	for(wn=0;wn<wcslen(wcs);wn++)
-		if(wcs[wn]=='\n' || (wcs[wn]=='\\' && wcs[wn+1]=='n'))
+		if(wcs[wn]=='\n' || (wcs[wn]=='\\' && wcs[wn+1]=='n' && strchr(" \t,.{[]1234567890",wcs[wn+2])))
 		{	wnl = wcs+wn;	break;	}
 	if(wnl)
 	{
 		wcl = new wchar_t[wn+1];	memcpy(wcl,wcs,wn*sizeof(wchar_t));
 		wcl[wn]=0;	Putsw(p, wcl, font, size, dir, sh);
 		wnl = wcs[wn]=='\n'?wnl+1:wnl+2;
+		if(*wnl<=' ')	wnl++;
 		nl_shift++;	Putsw(p, wnl, font, size, dir, sh);	nl_shift--;
 		return;
 	}
 
 	bool upside = ( (((_sx==-1) ^ (Org.y==Max.y || Org.z==Max.z)) && (dir=='x' || dir=='X')) ||
 					(((_sy==-1) ^ (Org.x==Max.x || Org.z==Max.z)) && (dir=='y' || dir=='Y')) ||
-					(((_st==-1) ^ (Org.x==0 || Org.z==1)) && (dir=='t' || dir=='T')) ||
+					(((_st==1)) && (dir=='t' || dir=='T')) ||
 					(((_sz==-1) ^ (Org.y==Max.y || Org.x==Max.x)) && (dir=='z' || dir=='Z')) );
 	mreal pp[6] = {p.x,p.y,p.z,p.x,p.y,p.z};
 	Arrow1 = Arrow2 = '_';
@@ -409,22 +409,21 @@ void mglGraphAB::Putsw(mglPoint p, const wchar_t *wcs, const char *font, mreal s
 	{
 	case 'x':
 	case 'X':
-		pp[3] += pp[3] ? fabs(pp[3])*.001 :(Max.x - Min.x)/100;	break;
+//		pp[3] += pp[3] ? fabs(pp[3])*.001 :(Max.x - Min.x)/100;	break;
+		pp[3] += (Max.x - Min.x)/1000;	break;
 	case 'y':
 	case 'Y':
 		if(TernAxis)	upside = !upside;
-		pp[4] += pp[4] ? fabs(pp[4])*.001 : (Max.y - Min.y)/10;	break;
+//		pp[4] += pp[4] ? fabs(pp[4])*.001 : (Max.y - Min.y)/10;	break;
+		pp[4] += (Max.y - Min.y)/1000;	break;
 	case 'z':
 	case 'Z':
 		if(TernAxis)	upside = !upside;
-		pp[4] -= 0;
-		pp[5] += pp[5] ? fabs(pp[5])*.001 :(Max.z - Min.z)/100;	break;
+//		pp[4] -= 0;		pp[5] += pp[5] ? fabs(pp[5])*.001 :(Max.z - Min.z)/100;	break;
+		pp[5] += (Max.z - Min.z)/1000;	break;
 	case 't':
 	case 'T':
-//		upside = !upside;
-		pp[3] += (Max.x - Min.x)/100;
-		pp[4] -= (Max.y - Min.y)/100;
-		pp[5] += 0;	break;
+		pp[3] += (Max.x - Min.x)/1000;	pp[4] -= (Max.y - Min.y)/1000;	break;
 	}
 	if(upside)	shift = -shift;
 	shift *= fsize/2;
@@ -486,7 +485,7 @@ void mglGraphAB::Putsw(mglPoint p, const wchar_t *wcs, const char *font, mreal s
 		}
 	}
 	zoomx1=x1;	zoomx2=x2;	zoomy1=y1;	zoomy2=y2;
-	yPos -= nl_shift*shift*1.25;
+	yPos -= nl_shift*0.11/PlotFactor*fsize;
 	fnt->Puts(wcs,font1,col);
 	Pop();	delete []font1;	EndGroup();
 }
@@ -501,6 +500,7 @@ void mglGraphAB::Legend(int n, wchar_t **text,char **style, mreal x, mreal y,
 	mreal pp[15], r=GetRatio(), rh, rw, s3=PlotFactor;
 	if(size<=0)	size = -size*FontSize;
 	if(!font || !(*font))	font="L";
+	char *pA, *ff = new char[strlen(font)+1];	strcpy(ff,font);
 	llen *= 1.5;
 
 	rh=(r<1?r:1.)*size/6.;	rw=(r>1?1./r:1.)*size/8.;
@@ -515,7 +515,13 @@ void mglGraphAB::Legend(int n, wchar_t **text,char **style, mreal x, mreal y,
 	}
 	w = (w + llen*1.1f);	// add space for lines
 
-	Push();	memcpy(B,B1,9*sizeof(mreal));
+	bool rel = true;
+	if((pA=strchr(ff,'A')))
+	{
+		*pA = 'L';
+		rel = false;
+	}
+	Push();	Identity(rel);	//	memcpy(B,B1,9*sizeof(mreal));
 	if(LegendBox)	// draw bounding box
 	{
 		pp[2] = pp[5] = pp[8] = pp[11] = pp[14] = s3-0.01;
@@ -540,13 +546,19 @@ void mglGraphAB::Legend(int n, wchar_t **text,char **style, mreal x, mreal y,
 		pp[0] = x+0.1f*llen;	pp[3] = x+0.9f*llen;	pp[6] = x+0.5f*llen;
 //		ScalePoint(pp[0],pp[1],pp[2]);	ScalePoint(pp[3],pp[4],pp[5]);	ScalePoint(pp[6],pp[7],pp[8]);
 
-		curv_plot(2,pp,0);
-		if(m)	Mark(pp[6],pp[7],pp[8],m);
+		pPos=0;	curv_plot(2,pp,0);
+		if(m)	for(int j=0;j<LegendMarks;j++)
+		{
+			pp[8] = s3;	pp[7] = y+i*h+0.45f*h;
+			pp[6] = x+0.1f*llen + (j+1)*0.8f*llen/(1.+LegendMarks);
+//			ScalePoint(pp[6],pp[7],pp[8]);
+			Mark(pp[6],pp[7],pp[8],m);
+		}
 		SelectPen(TranspType!=2 ? "k-1":"w-1");
-		Putsw(mglPoint(x+(style[i][0]!=0?llen:0), y+i*h+0.3f*h, s3), text[i], font, size);
+		Putsw(mglPoint(x+(style[i][0]!=0?llen:0), y+i*h+0.3f*h, s3), text[i], ff, size);
 	}
 	ScalePuts = true;
-	Pop();	EndGroup();
+	Pop();	EndGroup();	delete []ff;
 }
 //-----------------------------------------------------------------------------
 void mglGraphAB::colorbar(const mglData &vv, const mglColor *cs, int where, mreal x, mreal y, mreal w, mreal h)
@@ -792,8 +804,9 @@ unsigned char **mglGraphAB::GetRGBLines(long &w, long &h, unsigned char *&f, boo
 	return p;
 }
 //-----------------------------------------------------------------------------
+/* for 72pdi, FontSize=6 --> 26pt */
 void mglGraphAB::SetFontSizePT(mreal pt, int dpi)
-{	FontSize = pt*Height*0.0675/dpi;	}
+{	FontSize = 16.6154*pt/dpi;	}
 //-----------------------------------------------------------------------------
 void mglGraphAB::FindOptOrg(mreal ax[3], mreal ay[3], mreal az[3])
 {
@@ -881,6 +894,7 @@ mreal mglGraphAB::GetOrgX(char dir)
 		if(dir=='x')		res = ax[0];
 		else if(dir=='y')	res = ay[0];
 		else if(dir=='z')	res = az[0];
+		else if(dir=='t')	res = Min.x;
 		else res = B[6]>0 ? Max.x:Min.x;
 	}
 	return res;
@@ -896,6 +910,7 @@ mreal mglGraphAB::GetOrgY(char dir)
 		if(dir=='x')		res = ax[1];
 		else if(dir=='y')	res = ay[1];
 		else if(dir=='z')	res = az[1];
+		else if(dir=='t')	res = Min.y;
 		else res = B[7]>0 ? Max.y:Min.y;
 	}
 	return res;
@@ -911,6 +926,7 @@ mreal mglGraphAB::GetOrgZ(char dir)
 		if(dir=='x')		res = ax[2];
 		else if(dir=='y')	res = ay[2];
 		else if(dir=='z')	res = az[2];
+		else if(dir=='t')	res = Min.z;
 		else res = B[8]>0 ? Max.z:Min.z;
 	}
 	return res;
@@ -956,7 +972,7 @@ int mgl_draw_class(mglGraph *gr, void *p)
 {	return p ? ((mglDraw *)p)->Draw(gr) : 0;	}
 void mgl_reload_class(int next, void *p)
 {	if(p)	((mglDraw *)p)->Reload(next);	}
-void mglGraphAB::Window(int argc, char **argv, mglDraw *draw, const char *title, bool maximize)
+void mglGraphAB::Window(int argc, char **argv, const char *title, mglDraw *draw, bool maximize)
 {
 	Window(argc, argv, mgl_draw_class, title, draw, mgl_reload_class, maximize);
 }
