@@ -46,12 +46,10 @@ void wcstrim_mgl(wchar_t *str)
 	wchar_t *c = mgl_wcsdup(str);
 	unsigned long n=wcslen(str);
 	long k;
-	for(k=0;k<long(wcslen(str));k++)	//   
-		if(!isspace(str[k]))	break;
+	for(k=0;k<long(wcslen(str));k++)	if(!isspace(str[k]))	break;
 	wcscpy(c,&(str[k]));
 	n = wcslen(c);
-	for(k=n-1;k>=0;k--)	//   
-		if(!isspace(c[k]))		break;
+	for(k=n-1;k>=0;k--)	if(!isspace(c[k]))	break;
 	c[k+1] = 0;
 	wcscpy(str,c);	free(c);
 }
@@ -73,8 +71,7 @@ void mglGraph::DrawXGridLine(mreal t, mreal y0, mreal z0)
 	bool *tt = new bool[GridPnts+1];
 	for(i=0;i<GridPnts+1;i++)
 	{
-		pp[3*i]=t;//TernAxis ? t-t*i/30. : t;
-		pp[3*i+1]=y0;//TernAxis ? t*i/30. : y0;
+		pp[3*i]=t;		pp[3*i+1]=y0;
 		pp[3*i+2]=Min.z+(Max.z-Min.z)*i/GridPnts;
 		tt[i] = ScalePoint(pp[3*i],pp[3*i+1],pp[3*i+2]);
 	}
@@ -82,7 +79,7 @@ void mglGraph::DrawXGridLine(mreal t, mreal y0, mreal z0)
 	for(i=0;i<GridPnts+1;i++)
 	{
 		pp[3*i]=t;	pp[3*i+2]=z0;
-		pp[3*i+1]=Min.y+(Max.y-(TernAxis ? t:Min.y))*i/GridPnts;
+		pp[3*i+1]=Min.y+(Max.y-(TernAxis ? Min.y+t-Min.x:Min.y))*i/GridPnts;
 		tt[i] = ScalePoint(pp[3*i],pp[3*i+1],pp[3*i+2]);
 	}
 	curv_plot(GridPnts+1,pp,tt);
@@ -96,8 +93,8 @@ void mglGraph::DrawYGridLine(mreal t, mreal x0, mreal z0)
 	bool *tt = new bool[GridPnts+1];
 	for(i=0;i<GridPnts+1;i++)
 	{
-		pp[3*i]=x0;//TernAxis ? t*i/30. : x0;
-		pp[3*i+1]=t;//TernAxis ? t-t*i/30. : t;
+		pp[3*i]=TernAxis ? Max.x-t+Min.y:x0;
+		pp[3*i+1]=t;
 		pp[3*i+2]=Min.z+(Max.z-Min.z)*i/GridPnts;
 		tt[i] = ScalePoint(pp[3*i],pp[3*i+1],pp[3*i+2]);
 	}
@@ -105,7 +102,7 @@ void mglGraph::DrawYGridLine(mreal t, mreal x0, mreal z0)
 	for(i=0;i<GridPnts+1;i++)
 	{
 		pp[3*i+2]=z0;	pp[3*i+1]=t;
-		pp[3*i]=Min.x+(Max.x-(TernAxis ? t : Min.x))*i/GridPnts;
+		pp[3*i]=Min.x+(Max.x-(TernAxis ? Min.x+Min.y-t : Min.x))*i/GridPnts;
 		tt[i] = ScalePoint(pp[3*i],pp[3*i+1],pp[3*i+2]);
 	}
 	curv_plot(GridPnts+1,pp,tt);
@@ -142,17 +139,17 @@ void mglGraph::DrawTGridLine(mreal t, mreal z0)
 	bool *tt = new bool[GridPnts+1];
 	for(i=0;i<GridPnts+1;i++)
 	{
-		pp[3*i]= t*i/GridPnts;
-		pp[3*i+1]=t-t*i/GridPnts;
+		pp[3*i]= Min.x+t*i/GridPnts;
+		pp[3*i+1]=Min.y+t-t*i/GridPnts;
 		pp[3*i+2]=z0;
 		tt[i] = ScalePoint(pp[3*i],pp[3*i+1],pp[3*i+2]);
 	}
 	curv_plot(GridPnts+1,pp,tt);
 	for(i=0;i<GridPnts+1;i++)
 	{
-		pp[3*i]=t-t*i/GridPnts;
-		pp[3*i+1]=t*i/GridPnts;
-		pp[3*i+2]=z0;
+		pp[3*i]=Min.x;		pp[3*i+1]=Min.y+t;
+		pp[3*i+2]=Min.z+(Max.z-Min.z)*i/GridPnts;
+		tt[i] = ScalePoint(pp[3*i],pp[3*i+1],pp[3*i+2]);
 	}
 	curv_plot(GridPnts+1,pp,tt);
 	delete [] tt;	delete [] pp;
@@ -227,13 +224,13 @@ void mglGraph::Grid(const char *dir, const char *pen)
 			for(t=z0;t>=Min.z*FLT_EPS;t*=10)	DrawZGridLine(t,x0,y0);
 		}
 	}
-	if(strchr(dir,'t'))
+	if(strchr(dir,'t') && TernAxis)
 	{
 		z0 = isnan(OrgT.z) ? GetOrgZ('z') : OrgT.z;
 		z0 = z0 - ddz*floor((z0-Min.z)/ddz);
 		if(xnum)	for(int i=0;i<xnum;i++)		DrawTGridLine(xval[i],z0);
-		else if(dx)	for(t=Min.x;t<=Max.x;t+=ddx)	DrawTGridLine(t,z0);
-		else if(dy)	for(t=Min.y;t<=Max.y;t+=ddy)	DrawTGridLine(t,z0);
+		else if(dx)	for(t=Min.x;t<=Max.x;t+=ddx)	DrawTGridLine(t-Min.x,z0);
+		else if(dy)	for(t=Min.y;t<=Max.y;t+=ddy)	DrawTGridLine(t-Min.y,z0);
 	}
 	SelectPen("k-1");
 	EndGroup();
@@ -243,39 +240,41 @@ void mglGraph::Labelw(char dir,const wchar_t *text,mreal pos,mreal size,mreal sh
 {
 	mreal t, x0, y0, z0;
 	SelectPen("k-1");
+	char font[33]="C";
+	if(pos<-0.2)	font[0]='L';	if(pos>0.2)	font[0]='R';
+	strcat(font,FontDef);
 	x0 = GetOrgX(dir);	y0 = GetOrgY(dir);	z0 = GetOrgZ(dir);
 	if(size<=0)	size = -size*FontSize;
 	if(dir=='x')
 	{
 		if(dx)	t = (Min.x+Max.x+pos*(Max.x-Min.x))/2;
 		else	t = Min.x*pow(Max.x/Min.x, (pos+1)/2);
-		Putsw(mglPoint(t,y0,z0),text,FontDef,size,'X',shift);
+		Putsw(mglPoint(t,y0,z0),text,font,size,'X',shift);
 	}
 	if(dir=='y')
 	{
 		if(TernAxis)
 		{
-			t = (pos+1)/2;
-			Putsw(mglPoint(Max.x-t,t,z0),text,FontDef,size,'T',shift);
+			t = (Min.y+Max.y+pos*(Max.y-Min.y))/2;
+			Putsw(mglPoint(Max.x+Min.y-t,t,z0),text,font,size,'T',shift);
 		}
 		else
 		{
-			if(dx)	t = (Min.y+Max.y+pos*(Max.y-Min.y))/2;
+			if(dy)	t = (Min.y+Max.y+pos*(Max.y-Min.y))/2;
 			else	t = Min.y*pow(Max.y/Min.y, (pos+1)/2);
-			Putsw(mglPoint(x0,t,z0),text,FontDef,size,'Y',shift);
+			Putsw(mglPoint(x0,t,z0),text,font,size,'Y',shift);
 		}
 	}
 	if(dir=='z')
 	{
-		if(dx)	t = (Min.z+Max.z+pos*(Max.z-Min.z))/2;
+		if(dz)	t = (Min.z+Max.z+pos*(Max.z-Min.z))/2;
 		else	t = Min.z*pow(Max.z/Min.z, (pos+1)/2);
-		Putsw(mglPoint(x0,y0,t),text,FontDef,size,'Z',shift);
+		Putsw(mglPoint(x0,y0,t),text,font,size,'Z',shift);
 	}
 	if(dir=='t' && TernAxis)
 	{
-		if(dx)	t = (Min.x+Max.x-pos*(Max.x-Min.x))/2;
-		else	t = Max.x*pow(Min.x/Max.x, (pos+1)/2);
-		Putsw(mglPoint(x0,t,z0),text,FontDef,size,'Y',shift);
+		t = (Min.y+Max.y-pos*(Max.y-Min.y))/2;
+		Putsw(mglPoint(Min.x,t,Min.z),text,font,size,'Y',shift);
 	}
 }
 //-----------------------------------------------------------------------------
@@ -328,16 +327,16 @@ void _mgl_tick_text(mreal z, mreal z0, mreal d, mreal v, int kind, wchar_t str[6
 	if((kind&1) && z>z0)
 	{
 		int n1,n2;
-		mglprintf(str, 64, fabs(u)<1 ? L"@{(+%.2g)}" : L"@{(+%.3g)}",u);	n1=wcslen(str);
-		mglprintf(str, 64, L"@{(+%g)}",u);	n2=wcslen(str);
-		if(n1<n2)	mglprintf(str, 64, L"@{(+%.2g)}",u);
+		mglprintf(str, 64, fabs(u)<1 ? L"@{(+%.2g)}" : L"@{(+%.3g)}",u);
+		n1=wcslen(str);	mglprintf(str, 64, L"@{(+%g)}",u);	n2=wcslen(str);
+		if(n1<n2)	mglprintf(str, 64, fabs(u)<1 ? L"@{(+%.2g)}" : L"@{(+%.3g)}",u);
 	}
 	else
 	{
 		int n1,n2;
 		mglprintf(str, 64, fabs(u)<1 ? L"%.2g" :  L"%.3g",u);
 		n1=wcslen(str);	mglprintf(str, 64, L"%g",u);	n2=wcslen(str);
-		if(n1<n2)	mglprintf(str, 64, L"%.2g",u);
+		if(n1<n2)	mglprintf(str, 64, fabs(u)<1 ? L"%.2g" :  L"%.3g",u);
 	}
 }
 //-----------------------------------------------------------------------------
@@ -359,10 +358,9 @@ void mglGraph::DrawXTick(mreal x, mreal y0, mreal z0, mreal dy, mreal dz, int f)
 	if(*SubTStl && f)	SelectPen(SubTStl);
 	DrawTick(pp,f!=0);
 }
-void mglGraph::AxisX(bool text)
+void mglGraph::AxisX(bool text, const char *stl)
 {
-	mreal pp[3*31];
-	bool tt[31];
+	mreal pp[3*32];
 	wchar_t str[64];
 	long i;
 	mreal x0,y0,z0,x,ddx,ddy,ddz,v=0;
@@ -374,14 +372,21 @@ void mglGraph::AxisX(bool text)
 	if(ddz==0)	ddz=z0*10>fabs(Max.z-Min.z) ? 0.3*fabs(Max.z-Min.z) : z0*3;
 	if(y0>(Max.y+Min.y)/2)	ddy = -ddy;
 	if(z0>(Max.z+Min.z)/2)	ddz = -ddz;
-	SelectPen(TranspType!=2 ? "k-1":"w-1");
+	SelectPen(stl);
 	for(i=0;i<31;i++)	// draw axis line
 	{
 		pp[3*i+1]=y0;	pp[3*i+2]=z0;
 		pp[3*i]=Min.x+(Max.x-Min.x)*i/30.;
-		tt[i] = ScalePoint(pp[3*i],pp[3*i+1],pp[3*i+2]);
+		ScalePoint(pp[3*i],pp[3*i+1],pp[3*i+2]);
 	}
-	curv_plot(31,pp,tt);
+	if(stl[3])
+	{
+		pp[3*31+1]=y0;	pp[3*31+2]=z0;
+		pp[3*31]=Min.x+(Max.x-Min.x)*1.05;
+		ScalePoint(pp[3*31],pp[3*31+1],pp[3*31+2]);
+		curv_plot(32,pp,0);
+	}
+	else	curv_plot(31,pp,0);
 
 	if(xnum)	for(int i=0;i<xnum;i++)
 	{
@@ -406,7 +411,6 @@ void mglGraph::AxisX(bool text)
 			x0 = x0 - ddx*floor((x0-Max.x)/ddx+1e-3);
 		}
 		for(x=x0;x<=x1;x+=ddx)
-//		for(x=x0;fabs(x-Max.x)<fabs(Max.x-Min.x)*1.000001;x+=ddx)
 		{
 			DrawXTick(x,y0,z0,ddy,ddz);
 			if(text)
@@ -469,17 +473,19 @@ void mglGraph::AxisX(bool text)
 void mglGraph::DrawYTick(mreal y, mreal x0, mreal z0, mreal dx, mreal dz, int f)
 {
 	mreal pp[9],ff=TickLen/sqrt(1.+f*st_t);
-	pp[0]=x0+dx*ff;	pp[1]=y;	pp[2]=z0;
+	if(TernAxis)
+	{	pp[0]=x0+dx*ff;	pp[1]=y-dx*ff;	pp[2]=z0;	}
+	else
+	{	pp[0]=x0+dx*ff;	pp[1]=y;	pp[2]=z0;	}
 	pp[3]=x0;		pp[4]=y;	pp[5]=z0;
 	pp[6]=x0;		pp[7]=y;	pp[8]=z0+dz*ff;
 	if(*TickStl && !f)	SelectPen(TickStl);
 	if(*SubTStl && f)	SelectPen(SubTStl);
 	DrawTick(pp,f!=0);
 }
-void mglGraph::AxisY(bool text)
+void mglGraph::AxisY(bool text, const char *stl)
 {
-	mreal pp[3*31];
-	bool tt[31];
+	mreal pp[3*32];
 	wchar_t str[64];
 	long i;
 	mreal x0,y0,z0,y,ddx,ddy,ddz,v=0;
@@ -491,14 +497,21 @@ void mglGraph::AxisY(bool text)
 	if(ddz==0)	ddz=z0*10>fabs(Max.z-Min.z) ? 0.3*fabs(Max.z-Min.z) : z0*3;
 	if(x0>(Max.x+Min.x)/2)	ddx = -ddx;
 	if(z0>(Max.z+Min.z)/2)	ddz = -ddz;
-	SelectPen(TranspType!=2 ? "k-1":"w-1");
+	SelectPen(stl);
 	for(i=0;i<31;i++)	//  
 	{
 		pp[3*i]=x0;	pp[3*i+2]=z0;
 		pp[3*i+1]=Min.y+(Max.y-Min.y)*i/30.;
-		tt[i] = ScalePoint(pp[3*i],pp[3*i+1],pp[3*i+2]);
+		ScalePoint(pp[3*i],pp[3*i+1],pp[3*i+2]);
 	}
-	curv_plot(31,pp,tt);
+	if(stl[3])
+	{
+		pp[3*31]=x0;	pp[3*31+2]=z0;
+		pp[3*31+1]=Min.y+(Max.y-Min.y)*1.05;
+		ScalePoint(pp[3*31],pp[3*31+1],pp[3*31+2]);
+		curv_plot(32,pp,0);
+	}
+	else	curv_plot(31,pp,0);
 
 	if(ynum)	for(int i=0;i<ynum;i++)
 	{
@@ -592,10 +605,9 @@ void mglGraph::DrawZTick(mreal z, mreal x0, mreal y0, mreal dx, mreal dy, int f)
 	if(*SubTStl && f)	SelectPen(SubTStl);
 	DrawTick(pp,f!=0);
 }
-void mglGraph::AxisZ(bool text)
+void mglGraph::AxisZ(bool text, const char *stl)
 {
-	mreal pp[3*31];
-	bool tt[31];
+	mreal pp[3*32];
 	wchar_t str[64];
 	long i;
 	mreal x0,y0,z0,z,ddx,ddy,ddz,v=0;
@@ -607,14 +619,21 @@ void mglGraph::AxisZ(bool text)
 	if(ddy==0)	ddy=y0*10>fabs(Max.y-Min.y) ? 0.3*fabs(Max.y-Min.y) : y0*3;
 	if(x0>(Max.x+Min.x)/2)	ddx = -ddx;
 	if(y0>(Max.y+Min.y)/2)	ddy = -ddy;
-	SelectPen(TranspType!=2 ? "k-1":"w-1");
+	SelectPen(stl);
 	for(i=0;i<31;i++)	//  
 	{
 		pp[3*i]=x0;	pp[3*i+1]=y0;
 		pp[3*i+2]=Min.z+(Max.z-Min.z)*i/30.;
-		tt[i] = ScalePoint(pp[3*i],pp[3*i+1],pp[3*i+2]);
+		ScalePoint(pp[3*i],pp[3*i+1],pp[3*i+2]);
 	}
-	curv_plot(31,pp,tt);
+	if(stl[3])
+	{
+		pp[3*31]=x0;	pp[3*31+1]=y0;
+		pp[3*31+2]=Min.z+(Max.z-Min.z)*1.05;
+		ScalePoint(pp[3*31],pp[3*31+1],pp[3*31+2]);
+		curv_plot(32,pp,0);
+	}
+	else	curv_plot(31,pp,0);
 
 	if(znum)	for(int i=0;i<znum;i++)
 	{
@@ -702,22 +721,28 @@ void mglGraph::Axis(const char *dir, bool adjust)
 	if(adjust)	AdjustTicks(dir);
 	static int cgid=1;	StartGroup("Axis",cgid++);
 	bool text = strchr(dir,'_')==NULL;
+	const char *arr = "AKDTVISO";
+	char stl[5]={TranspType!=2?'k':'w', '-', '1', 0, 0};
+	for(unsigned i=0;i<strlen(arr);i++)
+		if(strchr(dir,arr[i]))	{	stl[3]=arr[i];	break;	}
+
 	if(TernAxis)
 	{
 		_sx = _sy = _st = 1;
-		AxisX(text);
-//		AxisY(text);
-		AxisT(text);
+		if(fabs(Max.x-Min.x) != fabs(Max.y-Min.y))
+			SetWarn(mglWarnTern,"Axis");
+		AxisX(text, stl);
+		AxisT(text, stl);
 	}
 	else
 	{
-		if(strchr(dir,'x'))	{	_sx = 1;	AxisX(text);	}
-		if(strchr(dir,'X'))	{	_sx = -1;	AxisX(text);	}
-		if(strchr(dir,'y'))	{	_sy = 1;	AxisY(text);	}
-		if(strchr(dir,'Y'))	{	_sy = -1;	AxisY(text);	}
+		if(strchr(dir,'x'))	{	_sx = 1;	AxisX(text, stl);	}
+		if(strchr(dir,'X'))	{	_sx = -1;	AxisX(text, stl);	}
+		if(strchr(dir,'y'))	{	_sy = 1;	AxisY(text, stl);	}
+		if(strchr(dir,'Y'))	{	_sy = -1;	AxisY(text, stl);	}
 	}
-	if(strchr(dir,'z'))	{	_sz = 1;	AxisZ(text);	}
-	if(strchr(dir,'Z'))	{	_sz = -1;	AxisZ(text);	}
+	if(strchr(dir,'z'))	{	_sz = 1;	AxisZ(text, stl);	}
+	if(strchr(dir,'Z'))	{	_sz = -1;	AxisZ(text, stl);	}
 	EndGroup();
 }
 //-----------------------------------------------------------------------------
@@ -961,7 +986,7 @@ void mglGraph::Legend(int n, wchar_t **text,char **style, int where,
 	Legend(n,text,style,(x+1)/2,(y+1)/2,font,size,llen);
 }
 //-----------------------------------------------------------------------------
-void mglGraph::Ternary(bool t)
+void mglGraph::Ternary(int t)
 {
 	static mglPoint x1(-1,-1,-1),x2(1,1,1),o(NAN,NAN,NAN);
 	static bool c = true;
@@ -978,17 +1003,16 @@ void mglGraph::Ternary(bool t)
 void mglGraph::DrawTTick(mreal y, mreal x0, mreal z0, mreal dx, mreal dz, int f)
 {
 	mreal pp[9],ff=TickLen/sqrt(1.+f*st_t);
-	pp[0]=x0-y-dx*ff;	pp[1]=y;	pp[2]=z0;
-	pp[3]=x0-y;			pp[4]=y;	pp[5]=z0;
-	pp[6]=x0-y;			pp[7]=y;	pp[8]=z0+dz*ff;
+	pp[0]=x0-y+Min.y-dx*ff;		pp[1]=y;	pp[2]=z0;
+	pp[3]=x0-y+Min.y;			pp[4]=y;	pp[5]=z0;
+	pp[6]=x0-y+Min.y;			pp[7]=y;	pp[8]=z0+dz*ff;
 	if(*TickStl && !f)	SelectPen(TickStl);
 	if(*SubTStl && f)	SelectPen(SubTStl);
 	DrawTick(pp,f!=0);
 }
-void mglGraph::AxisT(bool text)
+void mglGraph::AxisT(bool text, const char *stl)
 {
-	mreal pp[3*31];
-	bool tt[31];
+	mreal pp[3*32];
 	wchar_t str[64];
 	long i;
 	mreal z0=GetOrgZ('z'), y, ddx, ddy, ddz;
@@ -998,43 +1022,60 @@ void mglGraph::AxisT(bool text)
 	if(ddx==0)	ddx=(Max.x-Min.x)/2;
 	if(ddy==0)	ddy=(Max.y-Min.y)/2;
 	if(ddz==0)	ddz=(Max.z-Min.z)/2;
-	SelectPen(TranspType!=2 ? "k-1":"w-1");
+	SelectPen(stl);
 
 	for(i=0;i<31;i++)	//  
 	{
 		pp[3*i]=Max.x+(Min.x-Max.x)*i/30.;
 		pp[3*i+2]=z0;
 		pp[3*i+1]=Min.y+(Max.y-Min.y)*i/30.;
-		tt[i] = ScalePoint(pp[3*i],pp[3*i+1],pp[3*i+2]);
+		ScalePoint(pp[3*i],pp[3*i+1],pp[3*i+2]);
 	}
-	curv_plot(31,pp,tt);
+	if(stl[3])
+	{
+		pp[3*31]=Max.x+(Min.x-Max.x)*1.05;
+		pp[3*31+1]=Min.y+(Max.y-Min.y)*1.05;
+		pp[3*31+2]=z0;
+		ScalePoint(pp[3*31],pp[3*31+1],pp[3*31+2]);
+		curv_plot(32,pp,0);
+	}
+	else	curv_plot(31,pp,0);
 	for(i=0;i<31;i++)	//  
 	{
 		pp[3*i]=Min.x;
 		pp[3*i+1]=Max.y+(Min.y-Max.y)*i/30.;
 		pp[3*i+2]=z0;
-		tt[i] = ScalePoint(pp[3*i],pp[3*i+1],pp[3*i+2]);
+		ScalePoint(pp[3*i],pp[3*i+1],pp[3*i+2]);
 	}
-	curv_plot(31,pp,tt);
+	if(stl[3])
+	{
+		pp[3*31]=Min.x;	pp[3*31+2]=z0;
+		pp[3*31+1]=Max.y+(Min.y-Max.y)*1.05;
+		ScalePoint(pp[3*31],pp[3*31+1],pp[3*31+2]);
+		curv_plot(32,pp,0);
+	}
+	else	curv_plot(31,pp,0);
 
 	NSy = NSy<0 ? 0 : NSy;
-	for(y=0;y<=Max.y+(Max.y-Min.y)*1e-6;y+=ddy)
+	for(y=Min.y;y<=Max.y+(Max.y-Min.y)*1e-6;y+=ddy)
 	{
 		if(y>Min.y && y<Max.y)	DrawTTick(y,Max.x,z0,ddx,ddz);
 		mglprintf(str,64,L"%.2g",y);		wcstrim_mgl(str);
-		if(text)	Putsw(mglPoint(Max.x-y,y,z0),str,FontDef,FontSize,'t');
+		if(text)	Putsw(mglPoint(Max.x+Min.y-y,y,z0),str,FontDef,FontSize,'t');
 
-		if(y>Min.y && y<Max.y)	DrawYTick(Max.y-y,Min.x,z0,ddx,ddz);
-		mglprintf(str,64,L"%.2g",y);		wcstrim_mgl(str);
-		if(text)	Putsw(mglPoint(Min.x,Max.y-y,z0),str,FontDef,FontSize,'y');
+		if(y>Min.y && y<Max.y)	DrawYTick(Max.y+Min.y-y,Min.x,z0,ddx,ddz);
+		mreal yy = 1-Min.x-Min.y-Max.y+y;
+		yy = fabs(yy)<(Max.y-Min.y)*1e-6 ? 0:yy;
+		mglprintf(str,64,L"%.2g",yy);	wcstrim_mgl(str);
+		if(text)	Putsw(mglPoint(Min.x,Max.y+Min.y-y,z0),str,FontDef,FontSize,'y');
 	}
 	if(NSy>0)
 	{
 		ddy /= (NSy+1);	// subticks drawing
-		for(y=0;y<=Max.y;y+=ddy)
+		for(y=Min.y;y<=Max.y;y+=ddy)
 		{
 			DrawTTick(y,Max.x,z0,ddx,ddz,1);
-			DrawYTick(Max.y-y,Min.x,z0,ddx,ddz,1);
+			DrawYTick(Max.y+Min.y-y,Min.x,z0,ddx,ddz,1);
 		}
 	}
 }
