@@ -21,9 +21,15 @@
 #include <ctype.h>
 #include <math.h>
 #include <string.h>
+#include <zlib.h>
 #ifdef HAVE_HDF5
 #include <hdf5.h>
 #endif
+#ifdef HAVE_HDF4
+#define intf hdf4_intf
+#include <hdf/mfhdf.h>
+#undef intf
+#endif
 
 #ifndef WIN32
 #include <glob.h>
@@ -297,23 +303,30 @@ void mglData::Save(const char *fname,int ns) const
 	fclose(fp);
 }
 //-----------------------------------------------------------------------------
+char *mgl_read_gz(gzFile fp)
+{
+	long size=1024,n=0,m;
+	char *buf=(char*)malloc(size);
+	while((m=gzread(fp,buf+size*n,size))>0)
+	{
+		if(m<size)	{	buf[size*n+m]=0;	break;	}
+		n++;		buf=(char*)realloc(buf,size*(n+1));
+	}
+	return buf;
+}
+//-----------------------------------------------------------------------------
 bool mglData::Read(const char *fname)
 {
 	long l=1,m=1,k=1;
 	long nb,i;
-	FILE *fp = fopen(fname,"rb");
+	gzFile fp = gzopen(fname,"r");
 	if(!fp)
 	{
 		if(!a)	Create(1,1,1);
 		return	false;
 	}
-	fseek(fp,0,SEEK_END);
-	nb = ftell(fp);
-	char *buf = new char[nb+1];
-	fseek(fp,0,SEEK_SET);
-	memset(buf,0,nb*sizeof(char));
-	fread(buf,nb,1,fp);
-	fclose(fp);
+	char *buf = mgl_read_gz(fp);
+	nb = strlen(buf);	gzclose(fp);
 
 	bool first=false,com=false;
 	register char ch;
@@ -321,11 +334,10 @@ bool mglData::Read(const char *fname)
 	buf[i+1]=0;	nb = i;		// remove tailing spaces
 	for(i=0;i<nb-1 && !isn(buf[i]);i++)	// determine nx
 	{
+		if(buf[i]=='#')		while(!isn(buf[i]) && i<nb)	i++;
 		ch = buf[i];
-//		sp = (ch==' ' || ch=='\t');
-		if(ch=='#')		while(!isn(buf[i]) && i<nb)	i++;
-		if(!isspace(ch) && !first)	first=true;
-		if(first && (ch==' ' || ch=='\t') && !isspace(buf[i+1])) k++;
+		if(ch>' ' && !first)	first=true;
+		if(first && (ch==' ' || ch=='\t') && buf[i+1]>' ') k++;
 	}
 	first = false;
 	for(i=0;i<nb-1;i++)					// determine ny
@@ -350,8 +362,7 @@ bool mglData::Read(const char *fname)
 		}
 	}
 	else	for(i=0;i<nb-1;i++)	if(buf[i]=='\f')	l++;
-
-	delete []buf;
+	free(buf);
 	return Read(fname,k,m,l);
 }
 //-----------------------------------------------------------------------------
@@ -368,16 +379,11 @@ void mglData::Create(int mx,int my,int mz)
 bool mglData::Read(const char *fname,int mx,int my,int mz)
 {
 	if(mx<=0 || my<=0 || mz<=0)	return false;
-	FILE *fp = fopen(fname,"rt");
+	gzFile fp = gzopen(fname,"r");
 	if(!fp)	return false;
 	Create(mx,my,mz);
-
-	fseek(fp,0,SEEK_END);
-	long nb = ftell(fp);
-	char *buf = new char[nb+1];
-	fseek(fp,0,SEEK_SET);
-	memset(buf,0,nb);	fread(buf,nb,1,fp);
-	fclose(fp);
+	char *buf = mgl_read_gz(fp);
+	long nb = strlen(buf);	gzclose(fp);
 
 	register long i=0, j=0, k=0;
 	while(j<nb)
@@ -402,25 +408,21 @@ bool mglData::Read(const char *fname,int mx,int my,int mz)
 		char *s=buf+j;
 		while(buf[j]>' ' && j<nb)	j++;
 		buf[j]=0;
-		a[i] = atof(s);	i++;	if(i>=nx*ny*nz)	break;
+		a[i] = atof(s);
+		i++;	if(i>=nx*ny*nz)	break;
 	}
-	delete []buf;
+	free(buf);
 	return true;
 }
 //-----------------------------------------------------------------------------
 bool mglData::ReadMat(const char *fname,int dim)
 {
 	if(dim<=0 || dim>3)	return false;
-	FILE *fp = fopen(fname,"rt");
+	gzFile fp = gzopen(fname,"r");
 	if(!fp)	return false;
 	nx = ny = nz = 1;	NewId();
-
-	fseek(fp,0,SEEK_END);
-	long nb = ftell(fp);
-	char *buf = new char[nb+1];
-	fseek(fp,0,SEEK_SET);
-	memset(buf,0,nb);	fread(buf,nb,1,fp);
-	fclose(fp);
+	char *buf = mgl_read_gz(fp);
+	long nb = strlen(buf);	gzclose(fp);
 
 	register long i=0,j=0;
 	while(j<nb)
@@ -463,7 +465,7 @@ bool mglData::ReadMat(const char *fname,int dim)
 		if(i>=nx*ny*nz)	break;
 		while(buf[j]>' ' && j<nb)	j++;
 	}
-	delete []buf;
+	free(buf);
 	return true;
 }
 //-----------------------------------------------------------------------------
@@ -775,8 +777,7 @@ void mglData::Squeeze(int rx,int ry,int rz,bool smooth)
 			s += a[i1+nx*(j1+ny*k1)];
 		b[i+kx*(j+ky*k)] = s/dx*dy*dz;
 	}
-	delete []a;
-	a = b;
+	delete []a;	a=b;
 	nx = kx;  ny = ky;  nz = kz;	NewId();
 }
 //-----------------------------------------------------------------------------
@@ -910,6 +911,45 @@ void mglData::Fill(const char *eq, mglPoint r1, mglPoint r2, const mglData *v, c
 	}
 }
 //-----------------------------------------------------------------------------
+void mglData::ReadHDF4(const char *fname,const char *data)
+{
+#ifdef HAVE_HDF4
+	int32 sd = SDstart(fname,DFACC_READ), nn, i;
+	if(sd==-1)	return;	// is not a HDF4 file
+	char name[64];
+	SDfileinfo(sd,&nn,&i);
+	for(i=0;i<nn;i++)
+	{
+		int32 sds, rank, dims[32], type, attr, in[2]={0,0};
+		sds = SDselect(sd,i);
+		SDgetinfo(sds,name,&rank,dims,&type,&attr);
+		if(!strcmp(name,data))	// as I understand there are possible many datas with the same name
+		{
+			if(rank==1)			Create(dims[0]);
+			else if(rank==2)	Create(dims[1],dims[0]);
+			else if(rank==3)	Create(dims[3],dims[1],dims[0]);
+			else	continue;
+			if(type==DFNT_FLOAT32)
+			{
+				float *b = new float[nx*ny*nz];
+				SDreaddata(sds,in,0,dims,b);
+				for(long j=0;j<nx*ny*nz;j++)	a[j]=b[j];
+				delete []b;
+			}
+			if(type==DFNT_FLOAT64)
+			{
+				double *b = new double[nx*ny*nz];
+				SDreaddata(sds,in,0,dims,b);
+				for(long j=0;j<nx*ny*nz;j++)	a[j]=b[j];
+				delete []b;
+			}
+		}
+		SDendaccess(sds);
+	}
+	SDend(sd);
+#endif
+}
+//-----------------------------------------------------------------------------
 #ifdef HAVE_HDF5
 void mglData::SaveHDF(const char *fname,const char *data,bool rewrite) const
 {
@@ -951,13 +991,15 @@ void mglData::ReadHDF(const char *fname,const char *data)
 {
 	hid_t hf,hd,hs;
 	hsize_t dims[3];
-	long rank;
-	hf = H5Fopen(fname, H5F_ACC_RDONLY, H5P_DEFAULT);
+	long rank, res = H5Fis_hdf5(fname);
+	if(res<=0)	{	ReadHDF4(fname,data);	return;	}
+	hf = H5Fopen(fname, H5F_ACC_RDONLY, H5P_DEFAULT);	if(hf<0)	return;
 #ifndef H5_USE_16_API
 	hd = H5Dopen(hf,data,H5P_DEFAULT);
 #else
 	hd = H5Dopen(hf,data);
 #endif
+	if(hd<0)	return;
 	hs = H5Dget_space(hd);
 	rank = H5Sget_simple_extent_ndims(hs);
 	if(rank>0 && rank<=3)
@@ -1082,7 +1124,7 @@ bool mglData::ReadAll(const char *templ, bool as_slice)
 	delete []fname;		free(b);
 	return true;
 #else
-    return false;
+	return false;
 #endif
 }
 //-----------------------------------------------------------------------------
