@@ -3,17 +3,17 @@
  * Copyright (C) 2007 Alexey Balakin <balakin@appl.sci-nnov.ru>            *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
- *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
- *   (at your option) any later version.                                   *
+ *   it under the terms of the GNU Library General Public License as       *
+ *   published by the Free Software Foundation; either version 3 of the    *
+ *   License, or (at your option) any later version.                       *
  *                                                                         *
  *   This program is distributed in the hope that it will be useful,       *
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
  *   GNU General Public License for more details.                          *
  *                                                                         *
- *   You should have received a copy of the GNU General Public License     *
- *   along with this program; if not, write to the                         *
+ *   You should have received a copy of the GNU Library General Public     *
+ *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
@@ -119,6 +119,8 @@ mreal mgl_fit_base(mglFitData *fd, mreal *ini)
 	gsl_matrix_free (covar);
 	delete []x_init;
 	return res;
+#else
+	return 0.0;
 #endif
 }
 //-----------------------------------------------------------------------------
@@ -193,10 +195,10 @@ mreal mglGraph::FitS(mglData &fit, const mglData &x, const mglData &y, const mgl
 		fd.s = s.a+i*y.nx;
 		res = mgl_fit_base(&fd,in.a);
 		for(j=0;j<fd.m;j++)	val[var[j]-'a'] = in.a[j];
-		for(j=0;j<y.nx;j++)
+		for(j=0;j<fit.nx;j++)
 		{
-			val['x'-'a'] = x.a[j+(i%x.ny)*y.nx];
-			fit.a[j+i*y.nx] = fd.eq->Calc(val);
+			val['x'-'a'] = Min.x+j*(Max.x-Min.x)/(fit.nx-1);
+			fit.a[j+i*fit.nx] = fd.eq->Calc(val);
 		}
 		if(ini)	memcpy(ini,in.a,fd.m*sizeof(mreal));
 	}
@@ -235,10 +237,11 @@ mreal mglGraph::FitS(mglData &fit, const mglData &xx, const mglData &yy, const m
 		fd.a = z.a+i*z.nx*z.ny;		fd.s = s.a+i*z.nx*z.ny;
 		res = mgl_fit_base(&fd,in.a);
 		for(j=0;j<fd.m;j++)	val[var[j]-'a'] = in.a[j];
-		for(j=0;j<z.nx*z.ny;j++)
+		for(j=0;j<fit.nx*fit.ny;j++)
 		{
-			val['x'-'a'] = x.a[j];	val['y'-'a'] = y.a[j];
-			fit.a[j+i*z.nx*z.ny] = fd.eq->Calc(val);
+			val['x'-'a'] = Min.x+(j%fit.nx)*(Max.x-Min.x)/(fit.nx-1);
+			val['y'-'a'] = Min.y+(j/fit.nx)*(Max.y-Min.y)/(fit.ny-1);
+			fit.a[j+i*fit.nx*fit.ny] = fd.eq->Calc(val);
 		}
 		if(ini)	memcpy(ini,in.a,fd.m*sizeof(mreal));
 	}
@@ -270,21 +273,22 @@ mreal mglGraph::FitS(mglData &fit, const mglData &xx, const mglData &yy, const m
 	fd.var = var;	fd.m = strlen(var);
 	fd.eq = new mglFormula(eq);
 	mglData in(fd.m);
-	if(fit.nx<FitPnts || fit.ny<FitPnts)	fit.Create(FitPnts, FitPnts, z.nz);
+	if(fit.nx<FitPnts || fit.ny<FitPnts || fit.nz<FitPnts)	fit.Create(FitPnts, FitPnts, FitPnts);
 	mreal val[MGL_VS], res = -1;
-	for(long i=0;i<z.nz && i<fit.nz;i++)
+
+	if(ini)	in.Set(ini,fd.m);	else in.Fill(0,0);
+	fd.a = a.a+i*a.nx*a.ny*a.nz;
+	res = mgl_fit_base(&fd,in.a);
+	for(j=0;j<fd.m;j++)	val[var[j]-'a'] = in.a[j];
+	for(i=0;i<fit.nz;i++)	for(j=0;j<fit.nx*fit.ny;j++)
 	{
-		if(ini)	in.Set(ini,fd.m);	else in.Fill(0,0);
-		fd.a = a.a+i*a.nx*a.ny*a.nz;
-		res = mgl_fit_base(&fd,in.a);
-		for(j=0;j<fd.m;j++)	val[var[j]-'a'] = in.a[j];
-		for(j=0;j<a.nx*a.ny*a.nz;j++)
-		{
-			val['x'-'a'] = x.a[j];	val['y'-'a'] = y.a[j];
-			val['z'-'a'] = z.a[j];	fit.a[j] = fd.eq->Calc(val);
-		}
-		if(ini)	memcpy(ini,in.a,fd.m*sizeof(mreal));
+		val['x'-'a'] = Min.x+(j%fit.nx)*(Max.x-Min.x)/(fit.nx-1);
+		val['y'-'a'] = Min.y+(j/fit.nx)*(Max.y-Min.y)/(fit.ny-1);
+		val['z'-'a'] = Min.z+i*(Max.y-Min.y)/(fit.nz-1);
+		fit.a[j+fit.nx*fit.ny*i] = fd.eq->Calc(val);
 	}
+	if(ini)	memcpy(ini,in.a,fd.m*sizeof(mreal));
+
 	PrepareFitEq(res, eq,var,ini,print);
 	delete fd.eq;
 	return res;
@@ -378,6 +382,66 @@ void mglGraph::PutsFit(mglPoint p, const char *pre, const char *font, mreal size
 	delete []buf;
 }
 //-----------------------------------------------------------------------------
+void mglGraph::Hist(mglData &res, const mglData &x, const mglData &a)
+{
+	int n1=res.nx, nn=a.nx*a.ny*a.nz;
+	if(n1<FitPnts)	n1=FitPnts;
+	res.Create(n1);
+	if(nn!=x.nx*x.ny*x.nz)	{	SetWarn(mglWarnDim);	return;	}
+	register int i,j1;
+	for(i=0;i<nn;i++)
+	{
+		j1 = int(n1*(x.a[i]-Min.x)/(Max.x-Min.x));	if(j1<0 || j1>=n1)	continue;
+		res.a[j1] += a.a[i];
+	}
+}
+//-----------------------------------------------------------------------------
+void mglGraph::Hist(mglData &res, const mglData &x, const mglData &y, const mglData &a)
+{
+	int n1=res.nx, n2=res.ny, nn=a.nx*a.ny*a.nz;
+	if(n1<FitPnts) n1=FitPnts;	if(n2<FitPnts) n2=FitPnts;
+	res.Create(n1,n2);
+	if(nn!=x.nx*x.ny*x.nz || nn!=y.nx*y.ny*y.nz)
+	{	SetWarn(mglWarnDim);	return;	}
+	register int i,j1,j2;
+	for(i=0;i<nn;i++)
+	{
+		j1 = int(n1*(x.a[i]-Min.x)/(Max.x-Min.x));	if(j1<0 || j1>=n1)	continue;
+		j2 = int(n2*(y.a[i]-Min.y)/(Max.y-Min.y));	if(j2<0 || j2>=n2)	continue;
+		res.a[j1+n1*j2] += a.a[i];
+	}
+}
+//-----------------------------------------------------------------------------
+void mglGraph::Hist(mglData &res, const mglData &x, const mglData &y, const mglData &z, const mglData &a)
+{
+	int n1=res.nx, n2=res.ny, n3=res.nz, nn=a.nx*a.ny*a.nz;
+	if(n1<FitPnts) n1=FitPnts;	if(n2<FitPnts) n2=FitPnts;	if(n3<FitPnts) n3=FitPnts;
+	res.Create(n1,n2,n3);
+	if(nn!=x.nx*x.ny*x.nz || nn!=y.nx*y.ny*y.nz || nn!=z.nx*z.ny*z.nz)
+	{	SetWarn(mglWarnDim);	return;	}
+	register int i,j1,j2,j3;
+	for(i=0;i<nn;i++)
+	{
+		j1 = int(n1*(x.a[i]-Min.x)/(Max.x-Min.x));	if(j1<0 || j1>=n1)	continue;
+		j2 = int(n2*(y.a[i]-Min.y)/(Max.y-Min.y));	if(j2<0 || j2>=n2)	continue;
+		j3 = int(n3*(z.a[i]-Min.z)/(Max.z-Min.z));	if(j3<0 || j3>=n3)	continue;
+		res.a[j1+n1*(j2+n2*j3)] += a.a[i];
+	}
+}
+//-----------------------------------------------------------------------------
+void mgl_hist_x(HMGL gr, HMDT res, const HMDT x, const HMDT a)
+{	gr->Hist(*res, *x, *a);	}
+void mgl_hist_xy(HMGL gr, HMDT res, const HMDT x, const HMDT y, const HMDT a)
+{	gr->Hist(*res, *x, *y, *a);	}
+void mgl_hist_xyz(HMGL gr, HMDT res, const HMDT x, const HMDT y, const HMDT z, const HMDT a)
+{	gr->Hist(*res, *x, *y, *z, *a);	}
+void mgl_hist_x_(uintptr_t* gr, uintptr_t* res, uintptr_t* x, uintptr_t* a)
+{	_GR_->Hist(_DM_(res), _D_(x), _D_(a));	}
+void mgl_hist_xy_(uintptr_t* gr, uintptr_t* res, uintptr_t* x, uintptr_t* y, uintptr_t* a)
+{	_GR_->Hist(_DM_(res), _D_(x), _D_(y), _D_(a));	}
+void mgl_hist_xyz_(uintptr_t* gr, uintptr_t* res, uintptr_t* x, uintptr_t* y, uintptr_t* z, uintptr_t* a)
+{	_GR_->Hist(_DM_(res), _D_(x), _D_(y), _D_(z), _D_(a));	}
+//-----------------------------------------------------------------------------
 mreal mgl_fit_1(HMGL gr, HMDT fit, const HMDT y, const char *eq, const char *var, mreal *ini)
 {	return gr->Fit(*fit, *y, eq, var, ini);	}
 mreal mgl_fit_2(HMGL gr, HMDT fit, const HMDT z, const char *eq, const char *var, mreal *ini)
@@ -422,6 +486,7 @@ mreal mgl_fit_xyzas_d(HMGL gr, HMDT fit, const HMDT x, const HMDT y, const HMDT
 
 void mgl_puts_fit(HMGL gr, mreal x, mreal y, mreal z, const char *prefix, const char *font, mreal size)
 {	gr->PutsFit(mglPoint(x,y,z), prefix, font, size);	}
+const char *mgl_get_fit(HMGL gr)	{	return gr->GetFit();	}
 //-----------------------------------------------------------------------------
 mreal mgl_fit_1_(uintptr_t* gr, uintptr_t* fit, uintptr_t* y, const char *eq, const char *var, mreal *ini, int l, int n)
 {
