@@ -28,11 +28,6 @@
 #include <wchar.h>
 #include "mgl/mgl.h"
 #include "mgl/mgl_eval.h"
-#ifndef MGL_DEF_VIEWER
-//#define MGL_DEF_VIEWER "kuickshow"
-#define MGL_DEF_VIEWER "gthumb"
-//#define MGL_DEF_VIEWER "gwenview";
-#endif
 //-----------------------------------------------------------------------------
 const char *mglWarn[mglWarnEnd] = {"%s: data dimension(s) is incompatible",
 								"%s: data dimension(s) is too small",
@@ -67,6 +62,21 @@ mglColorID mglColorIds[] = {{'k', mglColor(0,0,0)},
 	{' ', mglColor(-1,-1,-1)},	{0, mglColor(-1,-1,-1)}	// the last one MUST have id=0
 };
 //-----------------------------------------------------------------------------
+bool mglTestMode=false;
+void mglTest(const char *str, ...)
+{
+	if(mglTestMode)
+	{
+		char buf[256];
+		va_list lst;
+		va_start(lst,str);
+		vsprintf(buf,str,lst);
+		va_end(lst);
+		printf("TEST: %s\n",buf);
+		fflush(stdout);
+	}
+}
+//---------------------------------------------------------------------------
 void mglGraph::RecalcBorder()
 {
 	if(!fx &&	!fy &&	!fz)
@@ -240,7 +250,7 @@ void mglGraph::SetScheme(const char *s, bool face)
 	if(s[0]==0)	return;
 	strcpy(last_style, s);
 	NumCol = 0;
-	for(i=0;i<NUM_COLOR;i++)	cmap[i] = NC;
+	for(i=0;i<MGL_CMAP_COLOR;i++)	cmap[i] = NC;
 	cmap[0] = mglColor(0,0,0);	cmap[1] = mglColor(1,1,1);
 	OnCoord = false;	SmoothColorbar = true;
 	for(i=0;i<strlen(s);i++)
@@ -254,7 +264,7 @@ void mglGraph::SetScheme(const char *s, bool face)
 			{	cmap[NumCol].Set(s[i],(s[i+1]-'0')/5.f);	i++;	}
 			else	cmap[NumCol].Set(s[i]);
 			NumCol++;
-			if(NumCol>=NUM_COLOR)	break;
+			if(NumCol>=MGL_CMAP_COLOR)	break;
 		}
 		else
 		{
@@ -263,7 +273,7 @@ void mglGraph::SetScheme(const char *s, bool face)
 			{
 				cmap[NumCol].Set(s[i]);
 				NumCol++;
-				if(NumCol>=NUM_COLOR)	break;
+				if(NumCol>=MGL_CMAP_COLOR)	break;
 			}
 		}
 	}
@@ -420,8 +430,8 @@ char mglGraph::SelectPen(const char *p)
 	{
 //		strcpy(last_style, p);
 		const char *col = "wkrgbcymhRGBCYMHWlenuqpLENUQP";
-		const char *stl = " -|;:ji";
-		const char *mrk = "*o+xsd.^v";
+		const char *stl = " -|;:ji=";
+		const char *mrk = "*o+xsd.^v<>";
 		const char *wdh = "123456789";
 		const char *arr = "AKDTVISO_";
 		for(unsigned i=0;i<strlen(p);i++)
@@ -443,11 +453,17 @@ char mglGraph::SelectPen(const char *p)
 		}
 		if(strchr(p,'#'))
 		{
+			if(mk=='.')	mk = 'C';
+			if(mk=='+')	mk = 'P';
+			if(mk=='x')	mk = 'X';
 			if(mk=='o')	mk = 'O';
 			if(mk=='d')	mk = 'D';
 			if(mk=='s')	mk = 'S';
 			if(mk=='^')	mk = 'T';
 			if(mk=='v')	mk = 'V';
+			if(mk=='<')	mk = 'L';
+			if(mk=='>')	mk = 'R';
+			if(mk=='*')	mk = 'Y';
 		}
 	}
 	Pen(c, st, BaseLineWidth*w);
@@ -458,7 +474,8 @@ char mglGraph::SelectPen(const char *p)
 //-----------------------------------------------------------------------------
 void mglGraph::SetPal(const char *colors)
 {
-	if(!colors || !colors[0])	colors = DefPal;	// restore default
+	if(!colors || !colors[0])	// restore default if only 1 color
+	{	if(NumPal<2)	colors = DefPal;	else	return;	}
 	memset(Pal,0,100*sizeof(mglColor));
 	memset(PalNames,0,101*sizeof(char));
 	int i,n = strlen(colors),k;
@@ -469,7 +486,7 @@ void mglGraph::SetPal(const char *colors)
 		PalNames[k] = colors[i];	Pal[k] = mglColor(colors[i]);	k++;
 	}
 	NumPal = k;	//	CurrPal = 0;
-	if(k==0)	SetPal(0);	// if no colors then set default palette
+	if(k<1)	SetPal(0);	// if no colors then set default palette
 }
 //-----------------------------------------------------------------------------
 mglColor mglGraph::GetPal()
@@ -489,7 +506,35 @@ void mglGraph::SubPlot(int nx,int ny,int m, mreal dx, mreal dy)
 	else	{	dx /= 2;	dy /= 2;	}
 	x1 = (mx+dx)/nx;		x2 = (mx+1+dx)/nx;
 	y2 = 1.f-(my+dy)/ny;	y1 = 1.f-(my+1+dy)/ny;
-	InPlot(x1,x2,y1,y2);
+	InPlot(x1,x2,y1,y2,false);
+}
+//-----------------------------------------------------------------------------
+void mglGraph::SubPlot(int nx,int ny,int m, const char *style)
+{
+	mreal x1,x2,y1,y2;
+	int mx = m%nx, my = m/nx;
+	x1 = float(mx)/nx;		x2 = float(mx+1)/nx;
+	y2 = 1.f-float(my)/ny;	y1 = 1.f-float(my+1)/ny;
+	InPlot(x1,x2,y1,y2,style);
+}
+//-----------------------------------------------------------------------------
+void mglGraph::InPlot(mreal x1,mreal x2,mreal y1,mreal y2, const char *st)
+{
+	if(!st)		{	InPlot(x1,x2,y1,y2,false);	return;	}
+	if(strchr(st,'T'))	{	y1*=0.9;	y2*=0.9;	}	// general title
+	bool r = !(strchr(st,'r') || strchr(st,'R') || strchr(st,'>') || strchr(st,'g'));
+	bool l = !(strchr(st,'l') || strchr(st,'L') || strchr(st,'<') || strchr(st,'g'));
+	bool u = !(strchr(st,'u') || strchr(st,'U') || strchr(st,'_') || strchr(st,'g'));
+	bool a = !(strchr(st,'a') || strchr(st,'A') || strchr(st,'^') || strchr(st,'g') || strchr(st,'t'));
+	// let use simplified scheme -- i.e. no differences between axis, colorbar and/or title
+	register mreal xs=(x1+x2)/2, ys=(y1+y2)/2, f1 = 1.3, f2 = 1.1;
+	if(r && l)	{	x2=xs+(x2-xs)*f1;	x1=xs+(x1-xs)*f1;	}
+	else if(r)	{	x2=xs+(x2-xs)*f1;	x1=xs+(x1-xs)*f2;	}
+	else if(l)	{	x2=xs+(x2-xs)*f2;	x1=xs+(x1-xs)*f1;	}
+	if(a && u)	{	y2=ys+(y2-ys)*f1;	y1=ys+(y1-ys)*f1;	}
+	else if(a)	{	y2=ys+(y2-ys)*f1;	y1=ys+(y1-ys)*f2;	}
+	else if(u)	{	y2=ys+(y2-ys)*f2;	y1=ys+(y1-ys)*f1;	}
+	InPlot(x1,x2,y1,y2,false);
 }
 //-----------------------------------------------------------------------------
 void mglGraph::Printf(mglPoint p,const char *str,...)
@@ -556,8 +601,9 @@ void mglGraph::ClearEq()
 //-----------------------------------------------------------------------------
 void mglGraph::DefaultPlotParam()
 {
+	LegendMarks = 1;
 	FontSize = 5;			BaseLineWidth = 1;
-	Ambient();				Ternary(false);
+	Ambient();				Ternary(0);
 	PlotId = "frame";		SelectPen("k-1");
 	SetScheme("BbcyrR");	SetPalette(MGL_DEF_PAL);
 	SetTicks('x');	SetTicks('y');	SetTicks('z');	SetTicks('c');
@@ -587,7 +633,7 @@ void mglGraph::DefaultPlotParam()
 	for(int i=0;i<10;i++)
 	{	Light(i, mglPoint(0,0,1));	Light(i,false);	}
 	Light(0,true);		Light(false);
-	InPlot(0,1,0,1);	Zoom(0,0,1,1);
+	InPlot(0,1,0,1,false);	Zoom(0,0,1,1);
 }
 //-----------------------------------------------------------------------------
 void mglGraph::SetTickStl(const char *stl, const char *sub)
@@ -780,7 +826,7 @@ void mglGraph::string_curve(long f,long n,mreal *pp,long *nn,const wchar_t *text
 // Pure virtual functions below are defined here for compatibility with MinGW
 void mglGraph::Pen(mglColor , char ,mreal ){}
 void mglGraph::Light(int ,bool ){}
-void mglGraph::Light(int , mglPoint , mglColor , mreal , bool ){}
+void mglGraph::Light(int , mglPoint , mglColor , mreal , bool, mreal ){}
 void mglGraph::Clf(mglColor ){}
 void mglGraph::InPlot(mreal ,mreal ,mreal ,mreal ,bool ){}
 //-----------------------------------------------------------------------------
@@ -807,8 +853,8 @@ void mglGraph::Rotate(mreal TetX,mreal TetZ,mreal TetY)
 //-----------------------------------------------------------------------------
 void mglGraph::Fog(mreal d, mreal dz)	{	FogDist=d;	FogDz = dz;	}
 //-----------------------------------------------------------------------------
-void mglGraph::Light(int n,mglPoint p, char c, mreal bright, bool infty)
-{	Light(n,p,mglColor(c),bright,infty);	}
+void mglGraph::Light(int n,mglPoint p, char c, mreal bright, bool infty, mreal ap)
+{	Light(n,p,mglColor(c),bright,infty,ap);	}
 //-----------------------------------------------------------------------------
 void mglGraph::Ambient(mreal bright)	{	AmbBr = bright;	}
 //-----------------------------------------------------------------------------
@@ -848,12 +894,12 @@ void mglGraph::SetFont(mglFont *f)
 //-----------------------------------------------------------------------------
 void mglGraph::Title(const wchar_t *str,const char *font,mreal size)
 {
-	Identity();
+	Push();	Identity();
 	mglFormula *ox=fx, *oy=fy, *oz=fz;
 	fx = fy = fz = NULL;
 	Text(mglPoint((Min.x+Max.x)*0.5, Max.y+(Max.y-Min.y)*0.15, (Min.z+Max.z)*0.5), str, font, size);
 	fx=ox;	fy=oy;	fz=oz;
-	RestoreM();
+	Pop();
 }
 //-----------------------------------------------------------------------------
 void mglGraph::Title(const char *str,const char *font,mreal size)
@@ -867,7 +913,7 @@ void mglGraph::Title(const char *str,const char *font,mreal size)
 //-----------------------------------------------------------------------------
 void mglGraph::Labelw(mreal x, mreal y, const wchar_t *text, const char *fnt, mreal size, bool rel)
 {
-	Identity(rel);
+	Push();	Identity(rel);
 	mglFormula *ox=fx, *oy=fy, *oz=fz;
 	fx = fy = fz = NULL;
 	char *f = new char[strlen(fnt)+1];
@@ -878,7 +924,7 @@ void mglGraph::Labelw(mreal x, mreal y, const wchar_t *text, const char *fnt, mr
 				Max.z), text, f, size);
 	delete []f;
 	fx=ox;	fy=oy;	fz=oz;
-	RestoreM();
+	Pop();
 }
 //-----------------------------------------------------------------------------
 void mglGraph::Label(mreal x, mreal y, const char *str, const char *fnt, mreal size, bool rel)
@@ -1039,7 +1085,7 @@ void mglGraph::ShowImage(const char *viewer, bool keep)
 	if(keep)
 	{
 		sprintf(cmd,"%s %s &", viewer,fname);
-		system(cmd);
+		if(system(cmd)==-1)	printf("Error to call external viewer\n");
 #ifdef WIN32
 //		sleep(2);
 		sprintf(cmd,"del %s", fname);
@@ -1053,7 +1099,7 @@ void mglGraph::ShowImage(const char *viewer, bool keep)
 	else
 		sprintf(cmd,"%s %s; rm %s", viewer,fname,fname);
 #endif
-	system(cmd);
+	if(system(cmd)==-1)	printf("Error to call external viewer\n");
 	delete []cmd;
 }
 //-----------------------------------------------------------------------------
@@ -1074,11 +1120,26 @@ void mglGraph::SetAutoRanges(mreal x1, mreal x2, mreal y1, mreal y2, mreal z1, m
 void mglGraph::Colorbar(const char *sch,int where)
 {
 	SetScheme(sch);
+	// ‘0’ - right, ‘1’ - left, ‘2’ - above, ‘3’ - under
+	if(sch && strchr(sch,'>'))	where = 0;
+	if(sch && strchr(sch,'<'))	where = 1;
+	if(sch && strchr(sch,'^'))	where = 2;
+	if(sch && strchr(sch,'_'))	where = 3;
+	if(sch && strchr(sch,'A'))	{	Push();	Identity();	}
 	Colorbar(where, where==0?1:0, where==2?1:0, 1, 1);
+	if(sch && strchr(sch,'A'))	Pop();
 }
 //-----------------------------------------------------------------------------
 void mglGraph::Colorbar(const mglData &v, const char *sch,int where)
-{	Colorbar(v,sch,where, where==0?1:0, where==2?1:0, 1, 1);	}
+{
+	if(sch && strchr(sch,'>'))	where = 0;
+	if(sch && strchr(sch,'<'))	where = 1;
+	if(sch && strchr(sch,'^'))	where = 2;
+	if(sch && strchr(sch,'_'))	where = 3;
+	if(sch && strchr(sch,'A'))	{	Push();	Identity();	}
+	Colorbar(v,sch,where, where==0?1:0, where==2?1:0, 1, 1);
+	if(sch && strchr(sch,'A'))	Pop();
+}
 //-----------------------------------------------------------------------------
 void mglGraph::Colorbar(int where, mreal x, mreal y, mreal w, mreal h)
 {
@@ -1107,11 +1168,11 @@ void mglGraph::Colorbar(const mglData &v, const char *sch, int where, mreal x, m
 	delete []c;
 }
 //-----------------------------------------------------------------------------
-void mglGraph::ColumnPlot(int num, int i)
+void mglGraph::ColumnPlot(int num, int i, mreal dd)
 {
 	mreal d = i/(num+PlotFactor-1);
 	mreal w = PlotFactor/(num+PlotFactor-1);
-	InPlot(0,1,d,d+w,true);
+	InPlot(0,1,d,d+w*(1-dd),true);
 }
 //-----------------------------------------------------------------------------
 void mglGraph::SetCoor(int how)
