@@ -110,7 +110,7 @@ void QMathGL::paintEvent(QPaintEvent *)
 //-----------------------------------------------------------------------------
 void QMathGL::resizeEvent(QResizeEvent *ev)
 {
-	if(autoResize)
+	if(autoResize && ev->size().width()>0 && ev->size().height()>0)
 	{	graph->SetSize(ev->size().width(), ev->size().height());	update();	}
 	else	resize(graph->GetWidth(), graph->GetHeight());
 }
@@ -190,16 +190,18 @@ void QMathGL::zoomOut()
 void QMathGL::update(mglGraph *gr)
 {
 	if(gr==0)	gr = graph;
-	if(gr==0 || draw_func==0)	return;
+	if(gr==0)	return;
+	if(draw_func==0)	{	refresh();	return;	}
 	if(gr!=graph || graph->ClfOnUpdate)	gr->DefaultPlotParam();
+	char *buf=new char[2048];	buf[0]=0;	gr->Message = buf;
+
 	gr->Alpha(alpha);	gr->Light(light);
 	gr->View(tet,phi);	gr->Org = mglPoint(NAN,NAN,NAN);
 	gr->Perspective(per);
 	gr->Zoom(x1,y1,x2,y2);
 	gr->DrawFace = !rotate;
-
-	char *buf=new char[2048];	buf[0]=0;	gr->Message = buf;
 	draw_func(gr, draw_par);
+
 	if(buf[0] != 0)	QMessageBox::warning(this, appName, buf);
 	gr->Message = 0;		delete []buf;
 	mousePos="";
@@ -430,24 +432,19 @@ void QMathGL::aboutQt()	{	QMessageBox::aboutQt(this, tr("About Qt"));	}
 void QMathGL::print()
 {
 	QPrinter *printer = new QPrinter;
+	printer->setOrientation(getRatio()>1 ? QPrinter::Landscape : QPrinter::Portrait);
 	QPrintDialog printDlg(printer, this);
 	if (printDlg.exec() == QDialog::Accepted)
 	{
+		QRectF r = printer->pageRect(QPrinter::Inch);
+		int d1 = int(pic.width()/r.width()), d2 = int(pic.height()/r.height());
+		int dpi = printer->resolution();
+		if(dpi<d1)	dpi=d1;		if(dpi<d2)	dpi=d2;
+		printer->setResolution(dpi);
+		
 		QPainter p;
 		if(!p.begin(printer))	return;	// paint on printer
-		QRect r = p.viewport();
-		int w = r.width(), h = r.height(), h1;
-		h1 = int(w/getRatio());
-		if(h1<h)	h = h1;	else	w = int(h*getRatio());
-		mglGraphZB gr(w, h);
-		if(w*h > 240000)	gr.BaseLineWidth = sqrt(w*h/2.4e5);
-		update(&gr);
-
-		uchar *grBuf=0;
-		QPixmap pic;
-		convertFromGraph(pic, &gr, &grBuf);
 		p.drawPixmap(0,0,pic);
-		delete []grBuf;
 	}
 	delete printer;
 }
@@ -579,22 +576,21 @@ void mglGraphQT::Window(int argc, char **argv, int (*draw)(mglGraph *gr, void *p
 	Wnd = new QMainWindow;	Wnd->resize(650,480);
 	Wnd->setWindowTitle(title);
 	scroll = new QScrollArea(Wnd);
-	QMGL = new QMathGL(Wnd);
+	QMGL = new QMathGL(Wnd);	makeMenu();
 	QMGL->setPopup(popup);	QMGL->setGraph(this);
 	QMGL->setDraw(draw, par);
-	makeMenu();	qApp->processEvents();
+	qApp->processEvents();
 	scroll->setWidget(QMGL);
 	Wnd->setCentralWidget(scroll);
 	QMGL->update();
-	if(maximize)
-	{	Wnd->showMaximized();	}
-	else	Wnd->show();
+	if(!maximize)	Wnd->show();
+	else	Wnd->showMaximized();
 }
 //-----------------------------------------------------------------------------
 HMGL mgl_create_graph_qt_dr(HMDR dr, const char *title)
 {
 	mglGraphQT *g = new mglGraphQT;
-	g->Window(0,0,dr,title);
+	g->Window(0,0,title,dr);
 	return g;
 }
 //-----------------------------------------------------------------------------
@@ -626,6 +622,7 @@ void *mgl_qt_tmp(void *)	{	mglQtRun();	return 0;	}
 	pthread_detach(tmp);
 }*/
 //-----------------------------------------------------------------------------
+void mgl_qt_run()	{	mglQtRun();	}
 void mgl_qt_run_()	{	mglQtRun();	}
 //-----------------------------------------------------------------------------
 #define TR	QObject::tr
